# clusterbgo.tcl     cluster card BGO channel control

#  created 2-2-96  VP  from cluster ge

set debug 0
set started 0

set frame CLUSTERBGO
set frametitle "Cluster BGO Channel Setup"
set icontitle  "ClusterBGO"

set chanlist {???}
set chanindex 0
set seglist {A B C D E F G H I J K L M N O P Q R S Sum}
#    WARNING  - order in  seglist is same as order of channels on the card
#    segindex 0 is segment A and is the first channel 
set segindex 0 

set defaultchanlist {0 1 2 3 4 5 6 7}

set cardtype ClusterBGO
set chantype BGO

#    The array "table" which is indexed by the "widget name" contains all the information
#    needed for the register/display item
#
#    Each element of "table" is a list
#
#       0.    name of global parameter used to hold current value of the register
#       1.    procedure to be used to convert the widget value into parameter value
#       2.    procedure to be used to convert the parameter value into a widget value
#       3.    name of the register
#       4.    procedure to be used to convert the register value into parameter value
#       5.    procedure to be used to convert the parameter value into a register value
#       6.    name of unit used for regsiter access
#       7.    widget type to be used for "by parameter" displays
#       8-10  parameters for the procedure creating "by parameter" displays
#              eg  if (7) is :slider then 8 is min value, 9 is max value and 10 is unit
#       11    for startup only - initial value of the parameter (1)
#       12    for startup only - inserted for this widget into the table "parameternames"

set table(ALLCHANS) {allchans int-widget->param param->int-widget nil
       nil nil nil nil nil nil nil 0 nil}
set table(ALLSEGS)  {allsegs  int-widget->param param->int-widget nil
       nil nil nil nil nil nil nil 0 nil}
set table(ENABLE) {enable int-widget->param param->int-widget "Enable"
       enable->param param->enable nil :checkbox nil nil nil 1 "Enable"}
set table(LASTCARD) {lastcard int-widget->param param->int-widget "LastCard"
       int-reg->param param->int-reg nil :checkbox nil nil nil 1 "Last Card"}
set table(THRESHE) {threshE int-widget->param param->int-widget "Energy.Thresh"
       int-reg->param param->int-reg nil :slider 0 65536 1 1 "Energy Threshold"}
set table(ADDRE) {addrE widget->param param->hex-widget "Energy.Addr"
       int-reg->param param->int-reg nil :text 6 nil nil 1 Energy Address"}
set table(SSE) {ssE int-widget->param param->int-widget "Energy.SSEnable"
       int-reg->param param->int-reg nil :checkbox nil nil nil 1 "Energy Sliding Scale"}
set table(READOUTE) {readoutE int-widget->param param->int-widget "Energy.ROEnable"
       int-reg->param param->int-reg nil :checkbox nil nil nil 1 "Energy Readout"}
set table(THRESHTAC) {threshTAC int-widget->param param->int-widget "TAC.Thresh"
       int-reg->param param->int-reg nil :slider 0 65536 1 1 "TAC Threshold"}
set table(ADDRTAC) {addrTAC widget->param param->hex-widget "TAC.Addr"
       int-reg->param param->int-reg nil :text 6 nil nil 1 "TAC Address"}
set table(SSTAC) {ssE int-widget->param param->int-widget "TAC.SSEnable"
       int-reg->param param->int-reg nil :checkbox nil nil nil 1 "TAC Sliding Scale"}
set table(READOUTTAC) {readoutTAC int-widget->param param->int-widget "TAC.ROEnable"
       int-reg->param param->int-reg nil :checkbox nil nil nil 1 "TAC Readout"}

set table(MBE) {mbe widget->param param->SAROC-widget "Energy.ShiftCtrl"
       int-reg->param param->int-reg nil :message nil nil nil 1 "Energy Shift Control"}
set table(MBTAC) {mbtac widget->param param->SAROC-widget "TAC.ShiftCtrl"
       int-reg->param param->int-reg nil :message nil nil nil 1 "TAC Shift Control"}

set table(FTSAMPLE) { ftsample slider-widget->param param->int-widget "FTSample"
       unit-reg->param param->unit-reg ns :slider 0 3200 12 1 "Fast Trigger Sample (ns)"}
set table(VALSAMPLE) {valsample slider-widget->param param->int-widget "ValSample"
       unit-reg->param param->unit-reg ns :slider 0 16000 64 1 "Validation Sample (ns)"}
set table(VETOADJ) {vetoadj slider-widget->param param->int-widget "VETO"
       unit-reg->param param->unit-reg ns :slider 0 16000 64 1 "VETO (ns)"}
set table(SPAREADJ) {spareadj slider-widget->param param->int-widget "SpareDac"
       unit-reg->param param->unit-reg ns :slider 0 25000 96 1 "Spare DAC"}
set table(DISCTHRESH) {discthresh slider-widget->param param->int-widget "DiscThresh"
       unit-reg->param param->unit-reg mV :slider 0 63 1 1 "Disc Threshold"}


set table(MODELCODE) {modcode str-widget->param param->hex-widget "VXI.ModelCode"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(SERIALNUM) {serialnum str-widget->param param->hex-widget "VXI.SerialNum"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(MODLEVEL) {modlevel str-widget->param param->hex-widget "VXI.ModLevel"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(MANID) {manid str-widget->param param->hex-widget "VXI.ManID"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(OFFSET) {offset str-widget->param param->hex-widget "VXI.Offset"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}


proc param->SAROC-widget {w v u} {
#    insert-debug "param->SAROC-widget $w $v $u"
    switch $v {
       0  {set s 16}
       1  {set s 14}
       2  {set s 13}
       3  {set s 12}
       default {set s ??}
    }
    return "$w :label $s"
}


#    register selection for parameter save function

set registersavepatterns {
   BGO[A-S].Energy.Thresh BGO[A-S].Energy.SSEnable BGO[A-S].Energy.ROEnable BGO[A-S].Energy.Addr
   BGO[A-S].TAC.Thresh    BGO[A-S].TAC.SSEnable    BGO[A-S].TAC.ROEnable    BGO[A-S].TAC.Addr
   BGO[A-S].DiscThresh    BGO[A-S].Enable
   BGOSum.Energy.Thresh   BGOSum.Energy.SSEnable   BGOSum.Energy.ROEnable
   BGOSum.DiscThresh      BGOSum.Enable
   BGOPattern.High.ROEnable BGOPattern.Low.ROEnable
   ValSample FTSample VETO SpareDac GIR.ROCR GIR.ValAck.TimeOut
}


# inspection tables must be in same order as entries in the :choice-stacks
# this is achieved by using these tables when generating the choices
# index number returned maps directly to the signal number to be written to 
# the Signal Select register

set insp_table(MUX-AI1) {"Insp.A1" AI}
set insp_table(MUX-AI2) {"Insp.A2" AI}
set insp_table(MUX-DI1) {"Insp.D1" DI}
set insp_table(MUX-DI2) {"Insp.D2" DI}
set insp_table(MUX-VI)  {"Insp.V"  VI}

set insp_menu(MUX-AI1) analogueinsptable
set insp_menu(MUX-AI2) analogueinsptable
set insp_menu(MUX-DI1) digitalinsptable
set insp_menu(MUX-DI2) digitalinsptable
set insp_menu(MUX-VI)  voltageinsptable

set inspection_widgetnames {MUX-AI1 MUX-AI2 MUX-DI1 MUX-DI2 MUX-VI}
set inspection_regnames    {Insp.A1 Insp.A2 Insp.D1 Insp.D2 Insp.V}

#    the inspection line tables are in parallel
#    for channel select * implies value determined by adc channel
#    for parameter select * implies disconnect

set analogueinsptable {Disconnected Time-Ampl SUMO 0V Gated-Int GISum}
set AI-ChanSelect {0 * 20 21 * 19}
set AI-Param      {* 1  1  1 2  2}
set AI-BackParam      {0 32 52 53 64 83}

set digitalinsptable {Disconnected
        TACS
        TACOR FTRINT FTSAM VALSAM VETO
        SEDO
        Fast_Trigger Inhibit Validation Veto OrValack OrDeadTime
        StartReadoutSAROC
        Renin(SAROC) SAROCDTready0 SAROCDTready1 SAROCDTready2 SAROCDTready3
        SAROCDTack_buf SAROCEmpty0 SAROCEmpty1 SAROCEmpty2 SAROCEmpty3
        SAROCLPASS
        CSA*(FifoChipSelect_A) CLKA(FifoClock_A) ENA(FifoEnablePort_A)
        AS*(VME) DS(VME) R/W*(VME) ModRoco
        CardRenin* BLTACK(Card) BLTACK(Bus) OE_Ident*
        CSB*(FifoChipSelect_B) CLKB(FifoClock_B) ENB(FifoEnablePort_B) OR_Fifo(EmptyFlag)
        FDTACK DTACK*(VME) CardRenout SysClk LT_Reset* Link5_Ack Reset_Fifo* OE_DSP_Run
        en_DMA_Fifo Link5_Clk en_DMA_Saroc* Counter
        Link_Start OE_Link5_out DMA-Grant DMA-Req Link5_Dat0 Link5_Dat2 Link5_Dat2 Link5_Dat3
}
set DI-ChanSelect {0
        *
        19 20 21 22 23
        *
        32 32 32 32 32 32 32 32
        32 32 32 32 32
        32 32 32 32 32
        32 32 32 32 32 32 32 32
        32 32 32 32 32 32 32 32
        32 32 32 32 32 32 32 32
        32 32 32 32
        32 32 32 32 32 32 32
}
set DI-Param      {*
        1
        1 1 1 1 1
        2
        1 2 3 4 5 6 7 8
        9 10 11 12 13
        14 15 16 17 18
        19 20 21 22 23 24 25 26
        27 28 29 30 31 32 33 34
        35 36 37 38 39 40 41 42
        43 44 45 46
        47 48 49 50 51 52 53
        
}
set DI-BackParam      {0
        32
        51 52 53 54 55
        64
        101 102 103 104 105 106 107 108
        109 110 111 112 113
        114 115 116 117 118
        119 120 121 122 123 124 125 126
        127 128 129 130 131 132 133 134
        135 136 137 138 139 140 141 142
        143 144 145 146
        147 148 149 150 151 152 153
}

set voltageinsptable {Disconnected Discr-Thresh FTRSPC VALSPC VETOC DAC24 SLSCV +2.5V 0V}
set VI-ChanSelect {0 * 20 21 22 23 24 25 26}
set VI-Param      {* 1  1  1  1  1  1  1  1}
set VI-BackParam  {0 32 52 53 54 55 56 57 58}

set digitalinspmenu {Disconnected
        TACS
        TACOR FTRINT FTSAM VALSAM VETO
        SEDO
        {"GIR lines #1" Fast_Trigger Inhibit Validation Veto OrValack OrDeadTime}
        {"SAROC lines"  StartReadoutSAROC
        Renin(SAROC) SAROCDTready0 SAROCDTready1 SAROCDTready2 SAROCDTready3
        SAROCDTack_buf SAROCEmpty0 SAROCEmpty1 SAROCEmpty2 SAROCEmpty3 SAROCLPASS}
        {"GIR lines #2" CSA*(FifoChipSelect_A) CLKA(FifoClock_A) ENA(FifoEnablePort_A)
        AS*(VME) DS(VME) R/W*(VME) ModRoco
        CardRenin* BLTACK(Card) BLTACK(Bus) OE_Ident*
        CSB*(FifoChipSelect_B) CLKB(FifoClock_B) ENB(FifoEnablePort_B) OR_Fifo(EmptyFlag)
        FDTACK DTACK*(VME) CardRenout SysClk LT_Reset* Link5_Ack Reset_Fifo* OE_DSP_Run
        en_DMA_Fifo Link5_Clk en_DMA_Saroc* Counter
        Link_Start OE_Link5_out DMA-Grant DMA-Req Link5_Dat0 Link5_Dat2 Link5_Dat2 Link5_Dat3}
}

#    these are common
set steplist1 "FTSAMPLE VALSAMPLE VETOADJ"
#    these are per segment
set steplist2 "DISCTHRESH"


loadsource gir
loadsource  common.vxi

#proc EG {args} {puts stdout "EG: ($args)" }

#    enable register needs special attention because 2 bits have to be written
proc enable->param {p v}   {global $p; set $p [expr [scan-int $v] & 1]}
proc param->enable {p v u} {
   if {$v == 0} then {return 0} else {return 3}
}

proc click arg {
   global debug userselection allchans allsegs
   if {$debug} then {puts stderr "proc click called with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]
   if {$c==":quit"} then {return [click-quit $f]}
   switch -glob $w {
      *REDISPLAY  {click-redisplay}
      FORMAT      {click-format}
      ENGINEERING  {click-$v}
      STEP        {click-step $v $i}

      GIR*         {click-GIRwindow $w $v}

      MUX-*  {click-inspection $w [lindex $arg 4]}

      CHANNEL {
        click-channel $v
        catch {send cluster click-channel $v}
        catch {send clusterge click-channel $v}
      }
      SEGMENT {click-segment $v}

      ALLCHANS  {click-whichchans $v}
      ALLSEGS   {click-whichsegs $v}

      PARAM {click-param $v}

      default   {
          if {$allsegs < 0 || $allchans < 0} then {set userselection 1}
             click-on $w $v
             set userselection 0
      }
   }
}

set electronicsframe "[self frame-name].electronics"

proc make-frame1 {} {
  global env chanlist seglist analogueinsptable digitalinspmenu voltageinsptable stepmenu stepmode

  :frame "[self frame-name]" :label "[self frame-title]" :show-footer T  :resize F \
      :icon-image "@$env(ICONHOME)/vxi_module" \
      :icon-name "[self icon-title]"

    :panel PANEL1 :layout T :padx 10 :pady 5
      :layout H; :row-gap 10; :col-gap 10
      :choice-stack CHANNEL :label "Current Channel:" :strings $chanlist :valuex 18c
      :choice-stack SEGMENT :label "Current Segment:" :strings $seglist
      :checkbox ENABLE :label "Enable:" :valuex 10c
      :next-row T
      :choice ALLCHANS :label "Act on Channels?" :valuex 18c :strings [list Current All Select] :value 0
      :choice ALLSEGS  :label "Act on Segments?" :valuex 18c :strings [list Current All Select] :value 0

    :panel PANEL2 :layout T :padx 10
     :layout H; :col-gap 10
      :hskip 35
      :message M0 :label bits      :bold T
      :message M1 :label SS        :bold T
      :message M2 :label RO        :bold T
      :message M3 :label Address   :bold T
      :hskip 50
      :message M4 :label Threshold :bold T
      :row-gap 5; :next-row T
      :message MME :label "Energy" :w 6
      :message MBE :label "??" :w 3 
      :checkbox SSE                  :label " " 
      :checkbox READOUTE             :label " "
      :text ADDRE                    :label " " :w 6   :read-only T 
      :slider THRESHE                :label " " :min 0 :max 65535
      :next-row T
      :message MMTAC :label "TAC" :w 6
      :message MBTAC :label "??" :w 3 
      :checkbox SSTAC                  :label " " 
      :checkbox READOUTTAC             :label " "
      :text ADDRTAC                    :label " " :w 6   :read-only T 
      :slider THRESHTAC                :label " " :min 0 :max 65535
      :next-row T
      :slider DISCTHRESH  :label "Discriminator Threshold (mV):"  :min 0 :max 63 :valuex 30c

    :panel PANEL3 :layout T :padx 10 :pady 10
     :layout V; :row-gap 5
      :slider FTSAMPLE   :label "Fast Trigger Sample (ns):"  :min 0 :max 3200   :valuex 30c
      :slider VALSAMPLE  :label "Validation Sample (ns):"    :min 0 :max 16000  :valuex 30c
      :slider VETOADJ    :label "VETO Adjustment (ns):"      :min 0 :max 16000  :valuex 30c
#      :slider SPAREADJ   :label "Spare DAC:"                 :min 0 :max 25000  :valuex 30c

      :vskip 10
      :layout H; :col-gap 10; :row-gap 10
      :next-row T
      :menubutton MUX-AI1 :label "Analogue Inspection 1:" :w 22 :menu $analogueinsptable
      :message MUX-AI1-USE   :label ""
      :next-row T
      :menubutton MUX-AI2 :label "Analogue Inspection 2:" :w 22 :menu $analogueinsptable
      :message MUX-AI2-USE   :label ""
      :next-row T
      :menubutton MUX-DI1 :label "Digital Inspection 1:"  :w 22 :menu $digitalinspmenu
      :message MUX-DI1-USE   :label ""
      :next-row T
      :menubutton MUX-DI2 :label "Digital Inspection 2:"  :w 22 :menu $digitalinspmenu
      :message MUX-DI2-USE   :label ""
      :next-row T
      :menubutton MUX-VI  :label "Voltage Inspection:"    :w 22 :menu $voltageinsptable
      :message MUX-VI-USE   :label ""

   :panel PANEL6  :layout B :padx 10 :after FOOTER
    :layout H
      :button REDISPLAY 
      :menubutton STEP      :label "[lindex $stepmenu $stepmode]"  :strings $stepmenu
      :hskip 15
#      :button FORMAT        :label "By Parameter"
      :menubutton ENGINEERING :label "Engineering"  :menu [list electronics software]
      GIRbutton
      :button HELP          :command click-help
   :show T
}


proc make-frame2 {} {
  global env parameternames

  :frame "[self frame-name]" :label "[self frame-title]" :show-footer T  :resize F \
      :icon-image "@$env(ICONHOME)/vxi_module" \
      :icon-name "[self icon-title]"

        :panel PARAMS :layout T :padx 5 :pady 5
         :layout H
          :choice-stack PARAM :label "Parameter:" :valuex 15c :strings  $parameternames
          :hskip 70
          :choice ALLCHANS :label "Act on Channels?" :valuex 18c :strings [list Current All Select] :value 0
          :choice ALLSEGS  :label "Act on Segments?" :valuex 18c :strings [list Current All Select] :value 0
   
        :panel PANEL :layout T :padx 5

        :panel BUTTONS :layout B :padx 5 :after FOOTER
         :layout H
          :button REDISPLAY 
          :button FORMAT       :label "By Channel"
          :hskip 80
          :button HELP         :command click-help

  :show T
}



proc make-electronics {} {
  global env electronicsframe electronicsvisible

  :frame "$electronicsframe" :label "Cluster BGO Electronics" :show-footer T :resize F :transient T

    :panel ELECTRONICS :layout T :padx 5 :pady 5
     :layout H; :row-gap 5
      :checkbox LASTCARD  :label "Last Card:"                            :valuex 20c
      :text MODELCODE     :label "Model Code:" :w 6  :read-only T        :valuex 20c
      :next-row T
      :text SERIALNUM     :label "Serial Number:" :w 6 :read-only T      :valuex 20c
      :next-row T
      :text MODLEVEL      :label "Modification Level:" :w 6 :read-only T :valuex 20c
      :next-row T
      :text MANID         :label "Manufacturer ID:" :w 6  :read-only T   :valuex 20c
      :next-row T

      :text OFFSET        :label "Offset Register:" :w 6 :read-only T    :valuex 20c
  
      :show T
   set electronicsvisible 1
}        


proc make-segSELframe {} {
  global segsel seglist

   :frame SEGSEL-FRAME :label "Cluster BGO Segment Selection"  :show-footer F :resize F :transient T
   :panel SEGSEL-PANEL :layout T
   :choice SEGSEL :exclusive F :strings $seglist :value $segsel
   :show T
}         

proc make-chanSELframe {} {
  global chansel chanlist

   :frame CHANSEL-FRAME :label "Cluster BGO Channel Selection"  :show-footer F :resize F :transient T
   :panel CHANSEL-PANEL :layout T
   :choice CHANSEL :exclusive F :strings $chanlist :value $chansel
   :show T
}         

