# ge.tcl---Setup frame for Ge card

#    March 26 1996  conversion to tk  VP
# Wed Jan 25 17:30:37 GMT 1995, DB
#   substituted $env(EXPBASE) for /eg
# Wed Nov 30 11:02:18 GMT 1994, DB
#   printing via print templates introduced
# 9-12-92 PHO
#	made enable default parameter for 'by parameter' format
# 8-12-92 PHO
#	swapped BDC on/off
# Sun Nov  8 19:09:34 GMT 1992, DB
#   speeded up make-slider-panel a smidgin by creating the sliders with the
#   panel unshown
# 23-10-92 PHO
#	put step on sliders [lindex table(xxx) 10]
# 13-10-92 PHO
#	put help on 'By Parameter' frame
#	included test generator attenuation
# 17-9-92 PHO
#	changed range of test generator to 0 - 7 MeV
#	changed range of CFD Threshold to 0 - 2.6 MeV
# Thu Sep 10 20:05:20 BST 1992, DB
#   fixed loss of presets:menu on returning from 'by parameter' format to 'by channel'
# 9-9-92 PHO
#	included feedback after writing register
# Thu Aug 20 12:12:16 BST 1992, DB
#   added PRESETS:button for accelerated inspection line assignment
#   made Act-on-all-Channels unipotent
# Fri Jul 10 20:10:25 BST 1992 DB
#   merged in code for 'By Channel' and 'By Parameter' formats

set debug 0
set started 0

set frame GE
set frametitle "Tapered Detector (Ge) Card Setup"
set icontitle Ge

set chanlist {???}
set chanindex 0

set defaultchanlist {0 1 2 3 4 5 6 7}

set cardtype Ge
set VXIGlobal ""

set presetslist {"UP4/UP4" "//CFDDirect/CFDDelay"}         


#    The array "table" which is indexed by the "widget name" contains all the information needed
#    for the register/display item
#
#    Each element of "table" is a list
#
#       0.    name of global parameter used to hold current value of the register
#       1.    procedure to be used to convert the widget value into parameter value
#       2.    procedure to be used to convert the parameter value into a widget value
#       3.    name of the register
#       4.    procedure to be used to convert the register value into parameter value
#       5.    procedure to be used to convert the parameter value into a register value
#       6.    name of unit used for register access
#       7.    widget type to be used for "by parameter" displays
#       8-10  parameters for the procedure creating "by parameter" displays
#              eg  if (7) is :slider then 8 is min value, 9 is max value and 10 is unit
#       11    for startup only - initial value of the parameter (1)
#       12    for startup only - inserted for this widget into the table "parameternames"


set table(CHANNEL) {channel widget->channel channel->widget nil
        nil nil nil nil nil nil nil 1 nil}
set table(ALLCHANS) {allchans int-widget->param param->int-widget nil
        nil nil nil nil nil nil nil 0 nil}
set table(AMP) {amp int-widget->param param->int-widget "ExtAmp"
       int-reg->param param->int-reg nil :choice Int Ext nil 1 "Ext Amp"}
set table(VETO) {veto int-widget->param param->int-widget "VetoLemo"
       int-reg->param param->int-reg nil :choice Bus Lemo nil 1 "Veto"}
set table(TFA) {tfagain int-widget->param param->int-widget "TFAGain"
       int-reg->param param->int-reg nil :choice Low High nil 1 "TFA Gain"}
set table(BDC) {bdc int-widget->param param->int-widget "BDC"
       int-reg->param param->int-reg nil :choice On Off nil 1 "BDC"}
set table(MODE) {mode int-widget->param param->int-widget "TestMode"
       int-reg->param param->int-reg nil :checkbox nil nil nil 1 "Test Mode"}
set table(ENABLE) {enable int-widget->param param->int-widget "Enable"
       int-reg->param param->int-reg nil :checkbox nil nil nil 1 "Enable"}
set table(CONFENETRES) {confenetres int-widget->param param->int-widget "ConFenetres"
       int-reg->param param->int-reg nil :checkbox nil nil nil 1 "Con Fenetres"}
set table(LASTCARD) {lastcard int-widget->param param->int-widget "LastCard"
       int-reg->param param->int-reg nil :checkbox nil nil nil 1 "Last Card"}
set table(MARKREJECT) {markreject int-widget->param param->int-widget "PileupReject"
       int-reg->param param->int-reg nil :choice Mark Reject nil 1 "Pileup"}
set CFDTHRESHLOW {cfdthresh slider-widget->param param->int-widget "CFD.Thresh"
       unit-reg->param param->unit-reg keV :slider 0 2600 1 1 "CFD Threshold (keV)"}
set CFDTHRESHHIGH {cfdthresh slider-widget->param param->int-widget "CFD.Thresh"
       unit-reg->param param->unit-reg keV :slider 0 825 1 1 "CFD Threshold (keV)"}
set table(CFDTHRESH) $CFDTHRESHHIGH
set table(CFDDELAY) {cfddelay slider-widget->param param->int-widget "CFD.Delay"
       unit-reg->param param->unit-reg ns :slider 0 200 1 1 "CFD Delay (ns)"}
set table(CFDWIDTH) {cfdwidth slider-widget->param param->int-widget "CFD.Width"
       unit-reg->param param->unit-reg ns :slider 0 200 1 1 "CFD Width (ns)"}
set table(POLEZERO) {polezero slider-widget->param param->int-widget "PoleZero"
       int-reg->param param->int-reg nil :slider 0 255 1 1 "Pole Zero"}
set table(THRESH20) {thresh20 int-widget->param param->int-widget "20MeV.Thresh"
       int-reg->param param->int-reg nil :slider 0 65536 1 1 "20 MeV Threshold"}
set table(ADDR20) {addr20 widget->param param->hex-widget "20MeV.Addr"
       int-reg->param param->int-reg nil :text 6 nil nil 1 "20 MeV Address"}
set table(SS20) {ss20 int-widget->param param->int-widget "20MeV.SSEnable"
       int-reg->param param->int-reg nil :checkbox nil nil nil 1 "20 MeV Sliding Scale"}
set table(READOUT20) {readout20 int-widget->param param->int-widget "20MeV.ROEnable"
       int-reg->param param->int-reg nil :checkbox nil nil nil 1 "20 MeV Readout"}
set table(THRESH4) {thresh4 int-widget->param param->int-widget "4MeV.Thresh"
       int-reg->param param->int-reg nil :slider 0 65536 1 1 "4 MeV Threshold"}
set table(ADDR4) {addr4 widget->param param->hex-widget "4MeV.Addr"
       int-reg->param param->int-reg nil :text 6 nil nil 1 "4 MeV Address"}
set table(SS4) {ss4 int-widget->param param->int-widget "4MeV.SSEnable"
       int-reg->param param->int-reg nil :checkbox nil nil nil 1 "4 MeV Sliding Scale"}
set table(READOUT4) {readout4 int-widget->param param->int-widget "4MeV.ROEnable"
       int-reg->param param->int-reg nil :checkbox nil nil nil 1 "4 MeV Readout"}
set table(THRESHFT) {threshft int-widget->param param->int-widget "FT.Thresh"
       int-reg->param param->int-reg nil :slider 0 65536 1 1 "FT TAC Threshold"}
set table(ADDRFT) {addrft widget->param param->hex-widget "FT.Addr"
       int-reg->param param->int-reg nil :text 6 nil nil 1 "FT TAC Address"}
set table(SSFT) {ssft int-widget->param param->int-widget "FT.SSEnable"
       int-reg->param param->int-reg nil :checkbox nil nil nil 1 "FT TAC Sliding Scale"}
set table(READOUTFT) {readoutft int-widget->param param->int-widget "FT.ROEnable"
       int-reg->param param->int-reg nil :checkbox nil nil nil 1 "FT TAC Readout"}
set table(THRESHCO) {threshco int-widget->param param->int-widget "CO.Thresh"
       int-reg->param param->int-reg nil :slider 0 65536 1 1 "CO TAC Threshold"}
set table(ADDRCO) {addrco widget->param param->hex-widget "CO.Addr"
       int-reg->param param->int-reg nil :text 6 nil nil 1 "CO TAC Address"}
set table(SSCO) {ssco int-widget->param param->int-widget "CO.SSEnable"
       int-reg->param param->int-reg nil :checkbox nil nil nil 1 "CO TAC Sliding Scale"}
set table(READOUTCO) {readoutco int-widget->param param->int-widget "CO.ROEnable"
       int-reg->param param->int-reg nil :checkbox nil nil nil 1 "CO TAC Readout"}
set table(FTSAMPLE) { ftsample slider-widget->param param->int-widget "FTSample"
       unit-reg->param param->unit-reg ns :slider 0 2000 8 1 "Fast Trigger Sample (ns)"}
set table(VALSAMPLE) {valsample slider-widget->param param->int-widget "ValSample"
       unit-reg->param param->unit-reg ns :slider 0 10000 40 1 "Validation Sample (ns)"}
set table(WATCHDOG) {watchdog slider-widget->param param->int-widget "Watchdog"
       unit-reg->param param->unit-reg ns :slider 0 16000 63 1 "LT Watchdog Timeout (ns)"}
set table(PDSGATE) {pdsgate slider-widget->param param->int-widget "PDSGate"
       unit-reg->param param->unit-reg ns :slider 0 10000 40 1 "PDS Gate (ns)"}

set table(MODELCODE) {modcode str-widget->param param->hex-widget "VXI.ModelCode"
       int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(SERIALNUM) {serialnum str-widget->param param->hex-widget "VXI.SerialNum"
       int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(MODLEVEL) {modlevel str-widget->param param->hex-widget "VXI.ModLevel"
       int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(MANID) {manid str-widget->param param->hex-widget "VXI.ManID"
       int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(OFFSET) {offset str-widget->param param->hex-widget "VXI.Offset"
       int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(TESTGEN) {testgen slider-widget->param param->int-widget "TestGen" 
   unit-reg->param param->unit-reg keV :slider 0 7000 1 1 "Test Generator (KeV)"}
set table(ATTN) {attn int-widget->param param->int-widget "TestGen.Attn"
       int-reg->param param->int-reg nil :checkbox nil nil nil 1 "Test Generator Attenuation"}


#    register selection for parameter save function

set registersavepatterns {
   ExtAmp VetoLemo TFAGain BDC Enable ConFenetres PileupReject
   CFD.Thresh CFD.Delay CFD.Width PoleZero 
   FTSample ValSample Watchdog PDSGate
   20MeV.Thresh  20MeV.SSEnable 20MeV.ROEnable
   4MeV.Thresh   4MeV.SSEnable  4MeV.ROEnable
   FT.Thresh     FT.SSEnable    FT.ROEnable
   CO.Thresh     CO.SSEnable    CO.ROEnable
}


# inspection tables must be in same order as entries in the :choice-stacks

set insp_table(MUX-AI1) {"Insp.A1" AI}
set insp_table(MUX-AI2) {"Insp.A2" AI}
set insp_table(MUX-DI1) {"Insp.D1" DI}
set insp_table(MUX-DI2) {"Insp.D2" DI}
set insp_table(MUX-VI)  {"Insp.V"  VI}

set insp_menu(MUX-AI1) analogueinsptable
set insp_menu(MUX-AI2) analogueinsptable
set insp_menu(MUX-DI1) digitalinsptable
set insp_menu(MUX-DI2) digitalinsptable
set insp_menu(MUX-VI)  voltageinsptable

set inspection_widgetnames {MUX-AI1 MUX-AI2 MUX-DI1 MUX-DI2 MUX-VI}
set inspection_regnames    {Insp.A1 Insp.A2 Insp.D1 Insp.D2 Insp.V}

set analogueinsptable \
 {"Disconnected" "TFAInput" "TFAOutput" "PoleZero" "UP4" "PDS" "BDC" "UP20" "TAC"}

set digitalinsptable  \
 {"Disconnected" "CFDDirect" "CFDDelay" "FTPulse" "FTSample" 
  "ValPulse" "ValSample" "PDSGate" "EncodeCommand" "EOC" "ROReady"  "ROAck" 
  "LastPass" "TrigReset" "ADCClock" "Pileup" "ORTM"}

set voltageinsptable \
 {"Disconnected" "CFD.Thresh" "CFD.Delay" "CFD.Width" "PoleZero" "PDSGate" "FTSample" 
  "ValSample" "StartRdOut" "Vref" "V-24" "V-18" "V-12"
  "V-6" "V-5.2" "V-2" "V0" "V+5VA" "V+5" "V+6" "V+12" "V+18" "V+20" "V+24" "TestGen"} 

#    these are common
set steplist1 "CFDWIDTH CFDDELAY FTSAMPLE VALSAMPLE WATCHDOG CFDTHRESH POLEZERO PDSGATE"
#    no per segment items
set steplist2 ""


loadsource common.vxi


proc click arg {
   global debug userselection allchans
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]
   if {$c==":quit"} then {return [click-quit $f]}
   switch -glob $w {
      REDISPLAY   {click-redisplay}
      ENGINEERING  {click-$v}
      FORMAT      {click-format}
      STEP        {click-step $v $i}

      CHANNEL     {click-channel $v}
      ALLCHANS    {click-whichchans $v}

      PARAM       {click-param $v}
      MUX-*       {click-inspection $w [lindex $arg 4]}

      default   {
          if {$allchans < 0} then {set userselection 1}
             click-on $w $v
             if {$w == "TFA"} then {act_on_tfagain}
             set userselection 0
      }
   }
}


set electronicsframe "[self frame-name].electronics"

proc make-frame1 {} {
  global env chanlist
  global analogueinsptable digitalinsptable voltageinsptable stepmenu stepmode

  :frame [self frame-name] :label "[self frame-title]" :show-footer T :resize F \
      :icon-image "@$env(ICONHOME)/vxi_module" \
      :icon-name "[self icon-title]"

   :panel P1 :layout T :padx 10 :pady 5
     :layout H; :row-gap 10; :col-gap 10

      :choice-stack CHANNEL :label "Current Channel:" :strings $chanlist :valuex 18c
      :hskip 150
      :checkbox ENABLE :label "Enable:" :valuex 10c
    :next-row T
      :choice ALLCHANS :label "Act on Channels?" :valuex 18c :strings [list Current All Select] :value 0

    :row-gap 10; :col-gap 50
    :next-row T
      :choice VETO :label "Veto:"  :strings [list "Bus" "Lemo"] :valuex 12c
      :choice MARKREJECT :label "Pileup & Anti-Compton:" :strings [list "Mark" "Reject"] :valuex 25c
    :next-row T; :col-gap 25
      :choice TFA  :label "TFA Gain:"  :strings [list "Low" "High"]  :valuex 12c 
      :choice AMP  :label "Amp:" :strings [list "Int" "Ext"] :valuex 12c
      :choice BDC :label "BDC:" :strings [list "On" "Off"]  :valuex 12c

   :panel P2 :layout T :padx 10
     :layout H; :col-gap 25
      :hskip 75
      :message M1  :label "SS"        :bold T
      :message M2  :label "RO"        :bold T
      :message M3  :label "Address"   :bold T
      :hskip 50
      :message M4  :label "Threshold" :bold T
   :row-gap 5; :next-row T
      :checkbox SS20      :label "20 MeV"  :valuex 12c
      :checkbox READOUT20 :label " "
      :text ADDR20        :label " " :w 6   :read-only T 
      :slider THRESH20    :label " " :min 0 :max 65535
   :next-row T
      :checkbox SS4       :label "4 MeV"  :valuex 12c
      :checkbox READOUT4  :label " "
      :text ADDR4         :label " " :w 6   :read-only T 
      :slider THRESH4     :label " " :min 0 :max 65535
   :next-row T
      :checkbox SSFT      :label "Fast Trigger" :valuex 12c
      :checkbox READOUTFT :label " " 
      :text ADDRFT        :label " " :w 6   :read-only T 
      :slider THRESHFT    :label " " :min 0 :max 65535
   :next-row T
      :checkbox SSCO      :label "Cross Over"  :valuex 12c
      :checkbox READOUTCO :label " "
      :text ADDRCO        :label " " :w 6   :read-only T 
      :slider THRESHCO    :label " " :min 0 :max 65535

   :panel P3 :layout T :padx 10 :pady 10
     :layout V; :row-gap 5
      :slider CFDWIDTH  :label "CFD Width (ns):" :min 0 :max 200   :valuex 30c
      :slider CFDDELAY  :label "CFD Delay (ns):" :min 0 :max 200   :valuex 30c
      :slider FTSAMPLE  :label "Fast Trigger Sample (ns):" :min 0 :max 2000   :valuex 30c
      :slider VALSAMPLE :label "Validation Sample (ns):" :min 0 :max 16000   :valuex 30c
      :slider WATCHDOG  :label "LT Watchdog Timeout (ns):" :min 0 :max 16000 :valuex 30c
      :slider PDSGATE   :label "PDS Gate (ns):" :min 0 :max 10000   :valuex 30c
      :slider CFDTHRESH :label "CFD Threshold (KeV):" :min 0 :max 2600 :valuex 30c
      :slider POLEZERO  :label "Pole Zero:"            :min 0 :max 255 :valuex 30c

      :vskip 10
      :layout H; :col-gap 10; :row-gap 10
      :next-row T
      :menubutton MUX-AI1 :label "Analogue Inspection 1:" :w 22 :menu $analogueinsptable
      :message MUX-AI1-USE   :label ""
      :next-row T
      :menubutton MUX-AI2 :label "Analogue Inspection 2:" :w 22 :menu $analogueinsptable
      :message MUX-AI2-USE   :label ""
      :next-row T
      :menubutton MUX-DI1 :label "Digital Inspection 1:"  :w 22 :menu $digitalinsptable
      :message MUX-DI1-USE   :label ""
      :next-row T
      :menubutton MUX-DI2 :label "Digital Inspection 2:"  :w 22 :menu $digitalinsptable
      :message MUX-DI2-USE   :label ""
      :next-row T
      :menubutton MUX-VI  :label "Voltage Inspection:"    :w 22 :menu $voltageinsptable
      :message MUX-VI-USE   :label ""

   :panel P4 :layout B :padx 10 :after FOOTER
     :layout H
      :button REDISPLAY
      :menubutton STEP      :label "[lindex $stepmenu $stepmode]"  :strings $stepmenu 
      :hskip 15
      :button FORMAT :label "By Parameter"
      :menubutton ENGINEERING :label "Engineering"  :menu [list electronics software]
      :button HELP   :command click-help
   
 :show T

}

proc make-frame2 {} {
  global env parameternames

  :frame "[self frame-name]" :label "[self frame-title]" :show-footer T  :resize F \
      :icon-image "@$env(ICONHOME)/vxi_module" \
      :icon-name "[self icon-title]"

   :panel P10 :layout T :padx 5 :pady 5
     :layout H
      :choice-stack PARAM :label "Parameter:" :valuex 15c :strings $parameternames
      :hskip 200
      :choice ALLCHANS :label "Act on Channels?" :valuex 18c :strings [list Current All Select] :value 0
   
   :panel PANEL :layout T :padx 5

   :panel P11 :layout B  :padx 5 :after FOOTER
     :layout H
      :button REDISPLAY
      :button FORMAT :label "By Channel"
      :hskip 80
      :button HELP :command click-help
   
   :show T
}

proc make-electronics {} {
  global env electronicsframe electronicsvisible

  :frame "$electronicsframe" :label "Ge Electronics" :resize F :transient T

   :panel P20 :layout T :padx 5 :pady 5
     :layout H; :row-gap 5
      :checkbox LASTCARD  :label "Last Card:"                            :valuex 22c
      :text MODELCODE     :label "Model Code:" :w 6  :read-only T        :valuex 22c
    :next-row T
      :checkbox MODE      :label "Test Mode:"                            :valuex 22c
      :text SERIALNUM     :label "Serial Number:" :w 6 :read-only T      :valuex 22c
    :next-row T
      :checkbox CONFENETRES :label "Controle Fenetres:"                  :valuex 22c
      :text MODLEVEL      :label "Modification Level:" :w 6 :read-only T :valuex 22c
    :next-row T
      :text MANID         :label "Manufacturer ID:" :w 6  :read-only T   :valuex 22c
    :next-row T
      :checkbox ATTN      :label "Test Gen Attenuation:"                 :valuex 22c
      :text OFFSET        :label "Offset Register:" :w 6 :read-only T    :valuex 22c
    :next-row T
      :slider TESTGEN     :label "Test Generator (KeV):" :min 0 :max 7000  :valuex 22c
   
  :show T
 set electronicsvisible 1
}

proc make-chanSELframe {} {
  global chansel chanlist

   :frame CHANSEL-FRAME :label "Ge Channel Selection"  :show-footer F :resize F :transient T
   :panel CHANSEL-PANEL :layout T
   :choice CHANSEL :exclusive F :strings $chanlist :value $chansel
   :show T
}  
