#    hist task code overlay for detector type "clover"


set lowchannel 0
set highchannel 25
set maxchannel 25
set qualifier 1

set genames  {20MeV 4MeV FT CO}
set bgonames {Energy TAC Pattern}
set qnames   {q0 q1 q2 q3}
set qualnames {0 1 2 3}

set rdoutnames {RdOut.EvNum}

set germanium 0
set bgo 0
set rdout 0

set segnames {A B C D}
set seg 15


proc make-overlaypanel {} {
   global segnames seg
   global genames bgonames rdoutnames qnames

    :layout V; :row-gap 10; :col-gap 0

      :message DET-M1  :label "Clover detector overlay applied"

      :number DET-LOW  :label "Low channel:"   :w 5 :valuex 15c
      :number DET-HIGH :label "High channel:"  :w 5 :valuex 15c

      :choice DET-SEG  :label "Segment:"       :valuex 15c :exclusive F :strings $segnames :value $seg

      :choice DET-QUAL :label "Qualifier"  :valuex 15c :exclusive F :strings $qnames :value 1
      :choice DET-GE   :label "ADC - Ge:"  :valuex 15c :exclusive F :strings $genames :value 0
      :choice DET-BGO  :label "Sum BGO:"   :valuex 15c :exclusive F :strings $bgonames :value 0

      :choice DET-RDOUT      :label "Read Out"     :valuex 15c  :exclusive F :strings $rdoutnames :value 0
}



#    this proc handles events from widgets created in the proc make-overlaypanel
#    The widgets DET-LOW; DET-HIGH and DET-QUAL are handled in the main click procedure
#    and are not required here

proc click-overlay {w v} {

   switch -glob $w {
      DET-GE           {click-adc $v germanium}
      DET-BGO          {click-adc $v bgo}
      DET-SEG          {click-segment $v}
      DET-RDOUT        {click-adc $v rdout}
      default      {puts stdout $arg}
   }
}

proc repaint_overlay {} {
   global spectrumlist
   global germanium bgo seg rdout qualifier

   if {[llength $spectrumlist] > 0} then {
       :modify DET-GE  :value 0
       :modify DET-BGO :value 0
       :modify DET-SEG :value 0
       :modify DET-QUAL :value 0
       :modify DET-RDOUT :value 0
   } else {
       :modify DET-GE  :value $germanium
       :modify DET-BGO :value $bgo
       :modify DET-SEG :value $seg
       :modify DET-QUAL :value $qualifier
       :modify DET-RDOUT :value $rdout
   }
}


proc get-spectrum-list {} {
        global spectrumlist
        global genames bgonames rdoutnames germanium bgo rdout
        global qualifier qualnames
        global seg segnames
        global lowchannel highchannel

        if {$spectrumlist != {}} then {return $spectrumlist};  # return names selected from spectrum list

        set qlist   [list-take $qualnames [bits-in $qualifier]]
        set seglist [list-take $segnames  [bits-in $seg]]

        set list {}

        for {set i $lowchannel} {$i<=$highchannel} {incr i} {
# append ge spectrum names
           foreach adc [list-take $genames [bits-in $germanium]] {
              foreach s $seglist {
                 foreach q $qlist {lappend list Clover$i.Ge$s.$adc.q$q}
              }
           }
# and append corresponding bgo spectrum names
           foreach adc [list-take $bgonames [bits-in $bgo]] {lappend list Clover$i.BGO.$adc}
# and append rdout spectra
           foreach q $qlist {foreach adc [list-take $rdoutnames [bits-in $rdout]] {lappend list Clover$i.$adc.q$q}}
        }
        return $list
}

proc range-of n {

    set fields [split $n "."]
    switch -glob [second $fields]  {
        RdOut     {return 0x10000}
        default   {return 0x2000}
    }
}

proc click-segment v {
   global seg spectrumlist
   set seg $v
   if {$spectrumlist!={}} then {paint-histlist; repaint}
}
