#    hist task code overlay for detector type "ecc"


set lowchannel  0
set highchannel 9
set maxchannel  9

set qualifier 1

set eccnames   {6MeV 20MeV TAC}
set girnames    {GIR.EvNum}

set qnames   {q0 q1 q2 q3}
set qualnames {0 1 2 3}

set ecc 0
set gir 0

set segnames {A B C D}
set seg 15

proc make-overlaypanel {} {
   global eccnames girnames
   global segnames seg qnames

      :layout V; :row-gap 10; :col-gap 20

      :message DET-M1   :label "ECC overlay applied"

      :number DET-LOW  :label "Low channel:"  :w 5 :valuex 15c
      :number DET-HIGH :label "High channel:" :w 5 :valuex 15c

      :choice DET-SEG  :label "Segment:"       :valuex 15c :exclusive F :strings $segnames :value $seg

      :layout H
      :hskip 10
      :choice DET-GIR  :label "GIR" :exclusive F :strings [list EvNum] :value 0
      :next-row T
      :choice DET-QUAL :label "Qualifier"  :valuex 15c :exclusive F :strings $qnames :value 1
      :next-row T
      :choice DET-ECC   :label "Items"   :valuex 15c  :exclusive F :strings $eccnames  :value 0
}



#    this proc handles events from widgets created in the proc make-overlaypanel
#    The widgets DET-LOW; DET-HIGH and DET-QUAL are handled in the main click procedure
#    and are not required here

proc click-overlay {w v} {

   switch -glob $w {
      DET-ECC       {click-adc $v ecc}
      DET-SEG          {click-segment $v}
      DET-GIR          {click-adc $v gir}
      default      {puts stdout $arg}
   }
}

proc repaint_overlay {} {
   global spectrumlist
   global ecc gir seg qualifier

   if {[llength $spectrumlist] > 0} then {
       :modify DET-ECC   :value 0
       :modify DET-SEG      :value 0
       :modify DET-QUAL     :value 0
       :modify DET-GIR :value 0
   } else {
       :modify DET-ECC   :value $ecc
       :modify DET-SEG      :value $seg
       :modify DET-QUAL     :value $qualifier
       :modify DET-GIR      :value $gir
   }
}


proc get-spectrum-list {} {
        global spectrumlist
        global eccnames ecc girnames gir
        global lowchannel highchannel
        global qualifier qualnames
        global seg segnames

        if {$spectrumlist != {}} then {return $spectrumlist};  # return names selected from spectrum list

        set qlist   [list-take $qualnames [bits-in $qualifier]]
        set seglist [list-take $segnames  [bits-in $seg]]

        set list {}

        for {set i $lowchannel} {$i<=$highchannel} {incr i} {
# append  spectrum names
            foreach adc [list-take $eccnames  [bits-in $ecc]]  {
              foreach s $seglist {
                 foreach q $qlist {lappend list ECC$i.$s.$adc.q$q}
              }
            }
# append GIR
            foreach adc [list-take $girnames [bits-in $gir]] {
                 foreach q $qlist {lappend list ECC$i.$adc.q$q}
            }
        }
        return $list
}

proc click-segment {v} {
   global spectrumlist
   global seg
   set seg $v
   if {$spectrumlist!={}} then {paint-histlist; repaint}
}

proc range-of n {

    set fields [split $n "."]
    if {[string match "ECC*" [first $fields]]} then {
        switch -glob [third $fields]  {
            EvNum       {return 0x10000}
            TAC       {return 0x10000}
            default   {return 16384}
        }
    }
}
