# tc.tcl---timing constraints checker

# Fri Oct  2 11:31:36 BST 1992, DB
#   inaugurated
#   has tighter validation constraint while Local Trigger LCA is iffy

set debug 0
set tcformat "
   (:command-frame TC :label 'Timing Constraints Checker' :show-footer T
           :w 650 :h 230 :pin-in T
      (:panel :x 0 :y 0 :w 650 :h 30 (:message :label 'Warnings:' :bold T))
      (:galley TC-GALLEY :x 0 :y 30 :rows 10 :cols 90)
      (:panel :x 0 :y 200 :w 650 :h 30
         (:choice TC-EXTRASEG :label 'Include Clover Segment E:' :strings ('Yes' 'No') :value 1)
         (:button TC-APPLY :label 'Run Check')
         (:button TC-HELP :x 500 :label 'Help...')
      )
    :show T)"          

set frame TC
set frametitle "Timing Constraints Checker"
set warned 0
set cloverstart 51
set maxclover [expr $cloverstart + 24]
set phase1start 1
set maxphase1 [expr $cloverstart - 1]
set basesegs {A B C D}
set extraseg E
set allsegs  [concat $basesegs $extraseg] 
set cloversegs $basesegs

proc set-footer f {
   global frame
   xv "($frame :left-footer '[quote $f]')"
   }

proc clear-footer {} {
   global frame
   xv "($frame :left-footer ' ')"
   }

proc eg-error e {error "$e: [eg "(sys :error $e ?message)"]"}

proc write-reg {r v} {
   set-footer "setting register $r to value $v..."
   set z [catch {eg "(register :name '$r'  :value '$v')"} m]
   if {$z!=0} then {eg-error "$m"}
   }

proc write-regs {r v} {
   set-footer "setting registers $r to value $v..."
   set z [catch {eg "(register :pattern '$r'  :value '$v')"} m]
   if {$z!=0} then {eg-error "$m"}
   }

proc init-reg {r} {
   set-footer "setting register $r to its default value..."
   set z [catch {eg "(register :name '$r'  !initialise)"} m]
   if {$z!=0} then {eg-error "$m"}
   }

proc init-regs {r} {
   set-footer "setting registers $r to their default values..."
   set z [catch {eg "(register :pattern '$r'  !initialise)"} m]
   if {$z!=0} then {eg-error "$m"}
   }

proc read-reg {r} {
   set-footer "reading register $r..."
   set z [catch {eg "(register :name '$r'  ?value)"} m]
   if {$z!=0} then {eg-error "$m"}
   return $m
   }

proc read-timing-reg {r} {
   set-footer "reading register $r..."
   set z [catch {eg "(register :name '$r'  ?value)"} m]
   if {$z!=0} then {eg-error "$m"}
   set z [scan $m "%f%s" v u]
   if {$z!=2} then {error "register $r returned no time unit in $m"}
   if {$u=="ns"} then {return $v}
   if {$u=="us"} then {return [expr $v*1000.0]}
   if {$u=="ms"} then {return [expr $v*1000000.0]}
   error "register $r returned unacceptable time unit $u"
   }

proc read-regs {r} {
   set-footer "reading registers $r..."
   set z [catch {eg "(register :pattern '$r'  ?names-and-values)"} m]
   if {$z!=0} then {eg-error "$m"}
   return $m
   }

proc warning s {
   global warned
   xv "(TC-GALLEY :insert '[quote $s]')"
   set warned 1
   }

proc zap-galley {} {
   xv "(TC-GALLEY :erase T)"
   }

proc abs x {
   if {$x<0} then {return [expr -1*$x]} else {return $x}
   }

proc min {a b} {if {$a<=$b} then {return $a} else {return $b}}

proc max {a b} {if {$a>=$b} then {return $a} else {return $b}}

proc first p {lindex $p 0}

proc trunc r {
   set i [string first "." $r]
   if {$i<0} then {return $r}
   incr i -1
   return [string range $r 0 $i]
   }

proc compare-numeric {a b} {
   global debug
   if {$debug} then {puts stdout "{$a} {$b}"}
   set pat "(\[^0-9\]+)(\[0-9\]+)"
   set za [regexp $pat $a a0 a1 a2 ]
   set zb [regexp $pat $b b0 b1 b2 ]
   if {$za==0 || $zb==0} then {return [string compare $a $b]}
   if {"$a"!="$a0" || "$b"!="$b0"} then {return [string compare $a $b]}
   set z [expr int($a2-$b2)]
   if {$z!=0} then {return $z}
   return [string compare $a $b]
   }

proc click-quit {} {xv "(TC :delete)"}

proc click-extraseg v {
   global cloversegs basesegs allsegs
   if {$v == 0} then {set cloversegs $allsegs}
   if {$v == 1} then {set cloversegs $basesegs}
}

proc click-apply {} {
   global warned phase1start maxphase1 cloverstart maxclover cloversegs
   clear-footer
   zap-galley
# ValOffset is non-zero while Local Trigger LCA is iffy, DB, Sept 92
   set ValOffset 1500
   set warned 0
   set z [catch {
      set TrigRwGe1Delay [read-timing-reg /frontend/Trig.RwGe1.Delay]
      set TrigRwGe1Width [read-timing-reg /frontend/Trig.RwGe1.Width]
      set TrigFTDelay [read-timing-reg /frontend/Trig.FT.Delay]
      set TrigFTWidth [read-timing-reg /frontend/Trig.FT.Width]
      set TrigValDelay [read-timing-reg /frontend/Trig.Val.Delay]
      set TrigValWidth [read-timing-reg /frontend/Trig.Val.Width]
      set TrigFTValDelay [read-timing-reg /frontend/Trig.FTVal.Delay]
      set-footer "Checking Fast Trigger and Validation widths..."
#test      set TrigFTWidth 50
      if {$TrigFTWidth < 200} then \
         {warning "FT width (${TrigFTWidth}ns) is less than 200ns\n"}
      if {$TrigValWidth-$ValOffset < 200} then \
         {warning "Validation width (${TrigValWidth}ns) is less than [expr $ValOffset+200]ns\n"}
      
      set minCFDDelay 9999
      set maxCFDDelay 0
#test      set maxCFDDelay 9999

#      set zz [catch {eg "(register :pattern '/frontend/\[Ge?,Ge??].FTSample' ?names-and-objects)"} list]
      set zz [catch {eg "(register :pattern '/frontend/Ge\[?,??].FTSample' ?names)"} list]
      if {$zz != 0} then {set list {}}
      set ll {}
#      foreach pair $list {lappend ll [trunc [first $pair]]}
      foreach reg $list { lappend ll [trunc $reg] }
      set list [lsort -command compare-numeric $ll]

      foreach gechan $list {
         set GeFTSample [read-timing-reg /frontend/$gechan.FTSample]
         set GeValSample [read-timing-reg /frontend/$gechan.ValSample]
#test         set GeFTSample 6000
#test         set GeValSample 6000
         set GeCFDDelay [read-timing-reg /frontend/$gechan.CFD.Delay]
         set minCFDDelay [min $minCFDDelay $GeCFDDelay]
         set maxCFDDelay [max $maxCFDDelay $GeCFDDelay]

         set-footer "Checking $gechan FT Sample..."
         set advisedGeFTSample [expr $GeCFDDelay+$TrigRwGe1Delay+$TrigFTDelay+\
0.5*$TrigFTWidth+109.0]
         set deltaGeFTSample [abs [expr $advisedGeFTSample-$GeFTSample]]
         if {$deltaGeFTSample > 0.25*$TrigFTWidth} then \
            {warning "$gechan FT Sample out of range: actual ${GeFTSample}ns, advised ${advisedGeFTSample}ns\n"}

         set-footer "Checking $gechan Validation Sample..."
         set advisedGeValSample [expr $GeCFDDelay+$TrigRwGe1Delay+$TrigFTValDelay+\
$TrigValDelay+$ValOffset+0.5*($TrigValWidth-$ValOffset)+227.0]
         set deltaGeValSample [abs [expr $advisedGeValSample-$GeValSample]]
         if {$deltaGeValSample > 0.25*($TrigValWidth-$ValOffset)} then \
            {warning "$gechan Validation Sample out of range: actual ${GeValSample}ns, advised ${advisedGeValSample}ns\n"}
         }

#      set zz [catch {eg "(register :pattern '/frontend/\[Clover??].Ge.FTSample' ?names-and-objects)"} list]
      set zz [catch {eg "(register :pattern '/frontend/Clover\[??].Ge.FTSample' ?names)"} list]
      if {$zz != 0} then {set list {}}
      set ll {}
#      foreach pair $list {lappend ll [trunc [first $pair]]}
      foreach reg $list { lappend ll [trunc $reg] }

      foreach clvchn $ll {
         set GeFTSample [read-timing-reg /frontend/$clvchn.Ge.FTSample]
         set GeValSample [read-timing-reg /frontend/$clvchn.Ge.ValSample]
#test         set GeFTSample 6000
#test         set GeValSample 6000
         foreach clvseg $cloversegs {
            set GeCFDDelay [read-timing-reg /frontend/$clvchn.Ge$clvseg.CFD.Delay]
            set minCFDDelay [min $minCFDDelay $GeCFDDelay]
            set maxCFDDelay [max $maxCFDDelay $GeCFDDelay]

            set-footer "Checking $clvchn (segment $clvseg) FT Sample..."
            set advisedGeFTSample [expr $GeCFDDelay+$TrigRwGe1Delay+$TrigFTDelay+\
0.5*$TrigFTWidth+109.0]
            set deltaGeFTSample [abs [expr $advisedGeFTSample-$GeFTSample]]
            if {$deltaGeFTSample > 0.25*$TrigFTWidth} then \
            {warning "$clvchn FT Sample out of range (segment $clvseg): actual ${GeFTSample}ns, advised ${advisedGeFTSample}ns\n"}

            set-footer "Checking $clvchn (segment $clvseg) Validation Sample..."
            set advisedGeValSample [expr $GeCFDDelay+$TrigRwGe1Delay+$TrigFTValDelay+\
$TrigValDelay+$ValOffset+0.5*($TrigValWidth-$ValOffset)+227.0]
            set deltaGeValSample [abs [expr $advisedGeValSample-$GeValSample]]
            if {$deltaGeValSample > 0.25*($TrigValWidth-$ValOffset)} then \
            {warning "$clvchn Validation Sample out of range (segment $clvseg): actual ${GeValSample}ns, advised ${advisedGeValSample}ns\n"}
            }
         }

      set-footer "Checking range of CFD Delays..."
      if {$maxCFDDelay-$minCFDDelay > $TrigRwGe1Width} then \
         {warning "Spread of CFD Delays (${minCFDDelay}ns..${maxCFDDelay}ns) exceeds Input Sumbus pulse width (${TrigRwGe1Width}ns)\n"}

      } m ]
   if {$z!=0} then {display-error-report self $m; error $m} \
              else {clear-footer}
   if {$warned} then {set-footer "Be careful!!! Your timing is somewhat iffy"}\
                else {set-footer "Congratulations!!! Your timing is impeccable"}
   }              
   
proc click-help {} {
   clear-footer
   send mb goto [transparent "Timing Checker"]
   }

proc click arg {
   global debug
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$f=="ER"} then {remove-eg-error-report self; return}
   if {$c==":quit"} then {click-quit; return}
   case $w in {
      TC-APPLY {click-apply}
      TC-EXTRASEG {click-extraseg $v}
      TC-HELP {click-help}
      default    {puts stdout $arg}
      }
   }

proc self op {
   global frame frametitle
   case $op in {
      frame-name {return $frame}
      frame-title {return $frametitle}
      }
   }

proc start args {
   global tcformat
   xv $tcformat
   }

proc finish args {click-quit}

register-interest tc
