

#  tdr32v.defn.tcl     TDR32V VXI card

proc {AddTDR32VModule}  {module slot}  {
   global registerbase cardtype nsockets
   global file

   set registerbase $cardtype#$module
   set file 0

   delete-vxi-registers $registerbase.*

   TDR32VVXIRegisters $slot
   TDR32VModuleRegisters $slot
}

proc {BuildTDR32VModule}  {}  {
   global registerbase
   global file

   set registerbase ""

   set f "tdr32v.defn.c"

   set z [catch {open $f w} file]
   if {$z != 0} then {return $file}
      
   TDR32VVXIRegisters %d
   TDR32VModuleRegisters %d
   
   close $file
   return 0
}


proc tdr32v_register {registername class attributes} {
   global registerbase
   global file
   
   if {$file == 0} {
       define-vxi-register $registerbase $class $registername "$attributes"
   } else {
       if {$registerbase == ""} then {
           puts $file "\{\"$registername\",   \"$class\",  \"$attributes\"\},"
       } else {
           puts $file "\{\"$registerbase.$registername\",   \"$class\",  \"$attributes\"\},"
       }
   }
}

proc TDR32VVXIRegisters {slot} {

   tdr32v_register  datal          xDig   "-s$slot -o0x0 -w32 -prw -fx"
   tdr32v_register  dataw          xDig   "-s$slot -o0x0 -w16 -prw -fx"
   tdr32v_register  datas          xSht   "-s$slot -o0x0 -w16 -prw -fx"

   tdr32v_register  VXI.ManID      xSht   "-s$slot -o0  -pro -w16 -fx -b0  -l12"
   tdr32v_register  VXI.AddrSpace  xSht   "-s$slot -o0  -pro -w16 -fd -b12 -l2"
   tdr32v_register  VXI.DevClass   xSht   "-s$slot -o0  -pro -w16 -fd -b14 -l2"
   tdr32v_register  VXI.ModelCode  xSht   "-s$slot -o2  -pro -w16 -fx -b0  -l12"
   tdr32v_register  VXI.ReqMem     xSht   "-s$slot -o2  -pro -w16 -fd -b12 -l4"
   tdr32v_register  VXI.Reset      xSht   "-s$slot -o4  -prw -w16 -fd -b0  -l1"
   tdr32v_register  VXI.Passed     xSht   "-s$slot -o4  -pro -w16 -fd -b2  -l1"
   tdr32v_register  VXI.Ready      xSht   "-s$slot -o4  -pro -w16 -fd -b3  -l1"
   tdr32v_register  VXI.ModID      xSht   "-s$slot -o4  -pro -w16 -fd -b14 -l1"
   tdr32v_register  VXI.A24A32     xSht   "-s$slot -o4  -pro -w16 -fd -b15 -l1"
   tdr32v_register  VXI.Offset     xSht   "-s$slot -o6  -pro -w16 -fx"
   tdr32v_register  VXI.Control    xSht   "-s$slot -o4  -prw -w16 -fx -b0 -l0 -i0xffc0"
   tdr32v_register  VXI.SerialNum  xSht   "-s$slot -o8  -pro -w16 -fx"
   tdr32v_register  VXI.ModLevel   xSht   "-s$slot -o10 -pro -w16 -fx"
}


proc TDR32VModuleRegisters {slot} {
   global registerbase
   
#    card control register
   tdr32v_register  CCR            xDig   "-s$slot -c0 -o0x3800 -w32 -prw -fx -b0 -l0 -i0x0"
   tdr32v_register  CCR.bit0       xDig   "-s$slot -c0 -o0x3800 -w32 -prw -fx -b0 -l1"
   tdr32v_register  CCR.Scaler.Blocked.Enable       xDig   "-s$slot -c0 -o0x3800 -w32 -prw -fx -b1 -l1"
   tdr32v_register  CCR.bit2       xDig   "-s$slot -c0 -o0x3800 -w32 -prw -fx -b2 -l1"
   tdr32v_register  CCR.bit3       xDig   "-s$slot -c0 -o0x3800 -w32 -prw -fx -b3 -l1"
   tdr32v_register  CCR.ADC.Go     xDig   "-s$slot -c0 -o0x3800 -w32 -prw -fx -b4 -l1"
   tdr32v_register  CCR.ADC.Mode   xDig   "-s$slot -c0 -o0x3800 -w32 -prw -fx -b5 -l2"
   tdr32v_register  CCR.Reset      xDig   "-s$slot -c0 -o0x3800 -w32 -prw -fx -b8 -l1"

#    card status register
   tdr32v_register  CSR            xDig   "-s$slot -c0 -o0x3900 -w32 -pro -fx -b0 -l0"
   tdr32v_register  CSR.bit0       xDig   "-s$slot -c0 -o0x3900 -w32 -pro -fx -b0 -l1"
   tdr32v_register  CSR.bit1       xDig   "-s$slot -c0 -o0x3900 -w32 -pro -fx -b1 -l1"
   
#    gate group registers

   for {set Ch 0} {$Ch <= 31} {incr Ch} {
       
       set address [format "0x%x" [expr 0x400 + (($Ch / 8) * 0x100)]]
       set offset  [expr ($Ch - (($Ch / 8) * 8)) * 4]
       
       tdr32v_register  Ch$Ch.Group.Mode1        xDig   "-s$slot -c0 -a$address -o$offset -w32 -prw -fx -b0 -l0 -i0"
       tdr32v_register  Ch$Ch.Group.Mode1.Gate   xDig   "-s$slot -c0 -a$address -o$offset -w32 -prw -fx -b0 -l3"
       for {set bit 4} {$bit <= 7} {incr bit} {
          tdr32v_register  Ch$Ch.Group.Mode1.bit$bit  xDig   "-s$slot -c0 -a$address -o$offset -w32 -prw -fd -b$bit -l1"
       }
       
       tdr32v_register  Ch$Ch.Group.Mode2        xDig   "-s$slot -c0 -a$address -o0x20 -w32 -prw -fx -b0 -l0 -i0"
       for {set bit 0} {$bit <= 7} {incr bit} {
          tdr32v_register  Ch$Ch.Group.Mode2.bit$bit   xDig   "-s$slot -c0 -a$address -o0x20 -w32 -prw -fd -b$bit -l1"
       }
       
       tdr32v_register  Ch$Ch.Group.Mode3        xDig   "-s$slot -c0 -a$address -o0x24 -w32 -prw -fx -b0 -l0 -i0"
       for {set bit 0} {$bit <= 3} {incr bit} {
          tdr32v_register  Ch$Ch.Group.Mode3.bit$bit   xDig   "-s$slot -c0 -a$address -o0x24 -w32 -prw -fd -b$bit -l1"
       }
   }
   
#    8 * 32 bit scalers
   for {set Ch 0} {$Ch <= 7} {incr Ch} {
       tdr32v_register  Scaler$Ch.Data       xDig   "-s$slot -c0 -a0x1000 -o[expr $Ch * 4] -w32 -pro -fd -b0 -l0"
   }
   
#    8  scaler gates
   for {set Ch 0} {$Ch <= 7} {incr Ch} {
       tdr32v_register  Scaler$Ch.Gate       xDig   "-s$slot -c0 -a0x1020 -o[expr $Ch * 4] -w32 -prw -fd -b0 -l0"
   }

#    8 * scaler enables
   for {set Ch 0} {$Ch <= 7} {incr Ch} {
       tdr32v_register  Scaler$Ch.Enable       xDig   "-s$slot -c0 -a0x1040 -o0x0 -w32 -prw -fd -b$Ch -l1"
   }

   tdr32v_register  Scaler.Blocked.Data      xDig   "-s$slot -c0 -a0x1044 -o0x0 -w32 -pro -fd -b0 -l0"
   tdr32v_register  Scalers.Xfer             xDig   "-s$slot -c0 -a0x1100 -o0x0 -w32 -pwo -fd -b0 -l0"
   tdr32v_register  Scalers.Clear            xDig   "-s$slot -c0 -a0x1200 -o0x0 -w32 -pwo -fd -b0 -l0"
   tdr32v_register  Scaler.Blocked.Clear     xDig   "-s$slot -c0 -a0x1300 -o0x0 -w32 -pwo -fd -b0 -l0"
   
#    Readout

   tdr32v_register  Readout.FIFO.L           xDig   "-s$slot -c0 -a0x2000 -o0x0 -w32 -pro -fx -b0 -l0"
   tdr32v_register  Readout.FIFO.M           xDig   "-s$slot -c0 -a0x2000 -o0x4 -w32 -pro -fx -b0 -l0"
   tdr32v_register  Readout.FIFO.H           xDig   "-s$slot -c0 -a0x2000 -o0x8 -w32 -pro -fx -b0 -l0"
   
   tdr32v_register  Readout.Events           xDig   "-s$slot -c0 -a0x2200 -o0x0 -w32 -pro -fd -b0 -l0"

   tdr32v_register  Readout.Status           xDig   "-s$slot -c0 -a0x2400 -o0x0 -w32 -pro -fx -b0 -l0"
   tdr32v_register  RSR.bit0                 xDig   "-s$slot -c0 -a0x2400 -o0x0 -w32 -pro -fd -b0 -l1"
   tdr32v_register  RSR.bit1                 xDig   "-s$slot -c0 -a0x2400 -o0x0 -w32 -pro -fd -b1 -l1"

   tdr32v_register  Readout.Control          xDig   "-s$slot -c0 -a0x2800 -o0x0 -w32 -prw -fx -b0 -l0"
   tdr32v_register  RCR.bit0                 xDig   "-s$slot -c0 -a0x2800 -o0x0 -w32 -prw -fd -b0 -l1"
   
#    Inspection Lines

   for {set L 1} {$L <= 4} {incr L} {
       tdr32v_register  Insp.D$L.Chip    xMux   "-s$slot -o[format "0x%x" [expr 0x4020 + (($L-1)*4)]] -w32 -pwo -b0 -l0 -dTDR32V -md$L"
       tdr32v_register  Insp.D$L.Signal  xDig   "-s$slot -o[format "0x%x" [expr 0x4000 + (($L-1)*4)]] -w32 -pwo -fd -b0 -l0"
       tdr32v_register  Insp.D$L.Select  xDig   "-s$slot -o[format "0x%x" [expr 0x4010 + (($L-1)*4)]] -w32 -pwo -fd -b0 -l0"
   }
   
   for {set L 1} {$L <= 2} {incr L} {
       tdr32v_register  Insp.A$L.Channel xDig   "-s$slot -o[format "0x%x" [expr 0x6004 + (($L-1)*8)]] -w32 -pwo -fd -b0 -l0"
       tdr32v_register  Insp.A$L.Signal  xMux   "-s$slot -o[format "0x%x" [expr 0x6000 + (($L-1)*8)]] -w32 -pwo -b0 -l0 -dTDR32V -ma$L"
   }

#    VETO LUT
   tdr32v_register  VETO_LUT_1  bVXI   "-s$slot -a0x800 -o0x0 -w32 -prw -fx -l64"
   tdr32v_register  VETO_LUT_2  bVXI   "-s$slot -a0x900 -o0x0 -w32 -prw -fx -l64"
   tdr32v_register  VETO_LUT_3  bVXI   "-s$slot -a0xa00 -o0x0 -w32 -prw -fx -l64"
   tdr32v_register  VETO_LUT_4  bVXI   "-s$slot -a0xb00 -o0x0 -w32 -prw -fx -l64"

#    Readout Ident and Control field RAM
   tdr32v_register  Ident&Control_RAM_1  bVXI   "-s$slot -a0x3000 -o0x0 -w32 -prw -fx -l64"
   tdr32v_register  Ident&Control_RAM_2  bVXI   "-s$slot -a0x3100 -o0x0 -w32 -prw -fx -l64"


#    Channel Readout Enable
   for {set Ch 0} {$Ch <= 31} {incr Ch} {
       tdr32v_register  Ch$Ch.ReadOutEnable  xDig "-s$slot -c[expr $Ch/4] -a0x5000 -z0x4 -o0x0 -w32 -prw -fd -b[expr $Ch - (($Ch/4) * 4)] -l1"
   }
   
#    Channel ADC FIFO full
   for {set Ch 0} {$Ch <= 31} {incr Ch} {
       tdr32v_register  Ch$Ch.ADC_FIFO_Full  xDig "-s$slot -c[expr $Ch/4] -a0x5020 -z0x4 -o0x0 -w32 -prw -fd -b[expr $Ch - (($Ch/4) * 4)] -l1"
   }

   tdr32v_register  TimeStampReLoad      xDig   "-s$slot -c0 -a0x7000 -o0x0 -w32 -prw -fx -b0 -l0"

   for {set Ch 0} {$Ch <= 31} {incr Ch} {
       tdr32v_register Ch$Ch.GateWidth       xDAC   "-s$slot -c$Ch -a0x0 -z0x4 -o0x0 -w32 -prw -qn -us -d0 -r0:50ns,0x1ff:25600ns"
   }

}

proc TDR32V_DAQ-setup   {args}   {
   set module [first $args]

    set m [string range $module 6 end]

#    set up the Ident and Control RAM

#    first 32 entries relate to ADC data

     set data ""
     for {set Ch 0} {$Ch <= 31} {incr Ch} {
         append data [format 0x%08x [expr (0xc000 + ($m << 5) + $Ch) << 16]]
         append data " "
     }

#    next 32 entries relate to pileup

     for {set Ch 0} {$Ch <= 31} {incr Ch} {
         append data [format 0x%08x [expr 0x80000000 + ($m << 24) + (1 << 20) + $Ch]]
         append data " "
     }

#    write these to the module
     write-vxi-register $module.Ident&Control_RAM_1 $data

#    next some special words

     set data ""
#    SYNC100
     append data [format 0x%08x [expr 0x80000000 + ($m << 24) + (4 << 20)]]
     append data " "
#    Resume
     append data [format 0x%08x [expr 0x80000000 + ($m << 24) + (3 << 20)]]
     append data " "
#    pause
     append data [format 0x%08x [expr 0x80000000 + ($m << 24) + (2 << 20)]]
     append data " "

#    fill the rest with unused items
     for {set Ch 3} {$Ch <= 63} {incr Ch} {
         append data [format 0x%08x [expr 0x80000000 + ($m << 24)]]
         append data " "
     }

#    write these to the module
    write-vxi-register $module.Ident&Control_RAM_2 $data
}

proc TDR32V_DAQ-reset   {args}   {
   set module [first $args]
#    actions to be determined
}

proc TDR32V_DAQ-halt   {args}   {
   set module [first $args]
#    actions to be determined
}

proc TDR32V_DAQ-go   {args}   {
   set module [first $args]
#    actions to be determined
}
