# vxicontrol.tcl---VXI Configuration & Control Frame
#     Version for use with Eurogam/Euroball

set debug 0
set EGreport 0
set started 0

loadsource vxicontrol.common

set frame VXICONTROL
set frametitle "VXI Control and Configuration Frame"
set icontitle  "VXI"

if {[info exists env(VXIServer)] == 0} then {set VXIServer VXI}  \
   else {set VXIServer $env(VXIServer)}

set RegisterServer $VXIServer

set CurrentVXIServer "undefined"
set TrigServer "undefined"
set TrigModule "Trig"
set Unknown "VXI"

set configdirectory [file join $env(MIDASAPPLBASE) config $VXIServer]
set NULL [file join $env(MIDASEXPBASE) NULL]
set NetVarRegDefn   "NetVarRegDefn"

set Trigpres 0
if {[lsearch [inform bf inquire tasklist] "vre #"] < 0}  then {set VREpres 0}  else {set VREpres 1}
if {[lsearch [inform bf inquire tasklist] "roco #"] < 0} then {set ROCOpres 0} else {set ROCOpres 1}

#    load moduledefinition list

loadsource vxicontrol.defn

set modulechoice {}
set modulenames  {}
set tasknames    {}
set confignames  {}

foreach item $moduledefinition {
  if {[fourth $item] != ""} then {
     append modulechoice " " \"[first $item]\"
     append modulenames  " " [second $item]
     append tasknames    " " [third $item]
     append confignames  " " [fourth $item]
  }
}

set VXICrateList {vxi1}
set VXIHostList {vxi1}
set VXIModuleList(vxi1) {}

set moduletype  [first $modulechoice]
set modulename  [first $modulenames]
set vxicrateindex 0
set vxicrate    [lindex $VXICrateList $vxicrateindex]
set vxihost     [lindex $VXIHostList $vxicrateindex]
set vximodule   ""

set vxislot 0
set modulenumber 0

set selection -1

set m {
       {{}            {}            {Module}  {VXI}  }
       {{Module Type} {Module Name} {Number} {Slot} }
      }

set datalist {}

#    configuration list
#      for each module need
#        1.   module type
#        2.   module name
#        3.   module number
#        4.   slot

set defaultlist {}


proc Find_Trigger {} {
  global TrigServer Trigpres TrigModule Unknown
  set TrigServer [VXImodule2crate Trig]
  set Trigpres 0
  if {$TrigServer != $Unknown} then {
#    system configuration has a module named Trig
     set TrigModule Trig
     set Trigpres [expr ![catch {read-vxi-register $TrigModule.VXI.Go} m]]
  }
  if {$Trigpres != 0} then {return}
  set TrigServer [VXImodule2crate TriggerMK2]
  if {$TrigServer != $Unknown} then {
#    system configuration has a module named TriggerMK2
     set TrigModule TriggerMK2
     set Trigpres [expr ![catch {read-vxi-register $TrigModule.VXI.Go} m]]
  }
  if {$Trigpres != 0} then {return}
  set Trigpres 0
}


#    called by SYS
#    returns 0 unless all crates have been software reset
proc VXI-ResetState {} {
   global VXICrateList
   foreach crate $VXICrateList {
      set server [VXIcrate2server $crate]
      set state [EG "register :name '/$server/NetVar_HasBeenReset' ?value"]
      if {$state == 0} then {return 0}
   }
#    if we get here all crates are RESET
   return 1
}

#    returns 0 unless all crates have been setup
proc VXI-SetupState {} {
   global VXICrateList
   foreach crate $VXICrateList {
      set server [VXIcrate2server $crate]
      set state [EG "register :name '/$server/NetVar_HasBeenSetup' ?value"]
      if {$state == 0} then {return 0}
   }
#    if we get here all crates are SETUP
   return 1
}

#    called by SYS
#    returns 0 unless all crates have been software restored
proc VXI-RestoreState {} {
   global VXICrateList
   foreach crate $VXICrateList {
      set server [VXIcrate2server $crate]
      set state [EG "register :name '/$server/NetVar_HasBeenRestored' ?value"]
      if {$state == 0} then {return 0}
   }
#    if we get here all crates are RESTORED
   return 1
}

#    DAQ-reset; DAQ-setup; DAQ-flush
#    called from SYS control
#    option argument
#      = 0    operate on ALL crates  - default
#      = 1    operate on only required crates

proc DAQ-reset {args} {
  global env VXIServer VXICrateList

  if {[catch {set x $env(EXPERIMENTBASE)}] != 0} then \
           {midas-warning "No Current Experiment!"; return}

  if {[llength $args] > 0 && [first $args] != 0} then {
     set m ""
     foreach crate $VXICrateList {
        set server [VXIcrate2server $crate]
        set state [EG "register :name '/$server/NetVar_HasBeenReset' ?value"]
        if {$state == 0} then {
           reset-VXI $server
        }
     }
  } else {
     set m [reset-VXI $VXIServer]
  }

  return "$m"
}

proc DAQ-setup {args} {
  global env VXIServer VXICrateList

  if {[catch {set x $env(EXPERIMENTBASE)}] != 0} then \
           {midas-warning "No Current Experiment!"; return}

  if {[llength $args] > 0 && [first $args] != 0} then {
     set m ""
     foreach crate $VXICrateList {
        set server [VXIcrate2server $crate]
        set state [EG "register :name '/$server/NetVar_HasBeenSetup' ?value"]
        if {$state == 0} then {
           setup-VXI $server
        }
     }
  } else {
     set m [setup-VXI $VXIServer]
  }

  return "$m"
}

proc DAQ-flush {args} {
  global env VXIServer

  if {[catch {set x $env(EXPERIMENTBASE)}] != 0} then \
           {midas-warning "No Current Experiment!"; return}

  set m [flush-VXI $VXIServer]

  return "$m"
}

proc DAQ-halt {} {
  global env VXIServer VXICrateList
  global ROCOpres VREpres Trigpres

#     called from SYS control
#     operates only on ALL crates

    if {[catch {set x $env(EXPERIMENTBASE)}] != 0} then \
           {midas-warning "No Current Experiment!"; return}

    Stop-VXI $VXIServer

    foreach crate $VXICrateList {halt-vxi-crate $crate}

# Reset ROCOs
    if {$ROCOpres != 0} then {write-vxi-registers ROCO\[?,??\].Reset 1}

#  disable VRE
  if {$VREpres} {
    set-footer "HALTing VREs"
    set-busy
    if {[inform vre vre-start] > 0} then {inform vre vre-disableDSP}
    clear-busy
    clear-footer
  }

  if {$Trigpres} then {return "VXI STOPped [date]"} \
    else {return "VXI STOPped -- Warning, no Trigger module present"}
}

proc DAQ-go {} {
  global env VXIServer VXICrateList CurrentVXIServer 
  global Trigpres VREpres ROCOpres

#     called from SYS control
#     operates only on ALL crates

   if {[catch {set x $env(EXPERIMENTBASE)}] != 0} then \
          {midas-warning "No Current Experiment!"; return}

   Stop-VXI $VXIServer

   foreach crate $VXICrateList {go-vxi-crate $crate}

# Enable ROCOs
    if {$ROCOpres != 0} then {write-vxi-registers ROCO\[?,??\].Enable 1}

#  enable VRE and zero event number
  if {$VREpres} {
   set-footer "GOing VREs"
   set-busy
    if {[inform vre vre-start] > 0} then {
       inform vre vre-enableDSP
       inform vre vre-zero-counts
    }
   clear-busy
   clear-footer
  }

   Go-VXI $VXIServer

   if {$Trigpres} then {return "VXI GOed [date]"} \
     else {return "VXI Goed -- Warning, no Trigger module present"}
}

proc make-cratelist {} {
  global VXICrateList VXIHostList VXIModuleList VXIServer
  global vxicrate vxihost vximodules vxicrateindex
  global configdirectory

#    find VXI crates available

   if {[catch {eg "(experiment ?resources)"} m]} then {
      midas-warning "no experiment connected!!!"
      set VXICrateList {vxi1}
      set VXIHostList {vxi1}
      set vxicrateindex 0
   } else {
      set VXICrateList [make-resource-list "name" $m $VXIServer]
      set VXIHostList  [make-resource-list "host" $m $VXIServer]
      if {$vxicrateindex >= [llength $VXICrateList]} then {set vxicrateindex 0}
   }
#    sort lists into sensible order
   set cratelist [lsort $VXICrateList]
   set hostlist {}
   foreach c $cratelist {
      lappend hostlist [lindex $VXIHostList [lsearch $VXICrateList $c]]
   }
   set VXICrateList $cratelist
   set VXIHostList $hostlist
   set vxicrate    [lindex $VXICrateList $vxicrateindex]
   set vxihost     [lindex $VXIHostList $vxicrateindex]
   foreach crate $VXICrateList {
      if {![info exists VXIModuleList($crate)]} then {set VXIModuleList($crate) ""}
   }
   set vximodules  $VXIModuleList($vxicrate)
   :modify CRATE :strings $VXICrateList :value $vxicrateindex
   :modify HOST  :value $vxihost
   :modify CONFIG :value [file join  $configdirectory $vxihost]
   click-load
   paint-datalist
   repaint
}

proc make-resource-list {type rlist resource} {
   set ll {}
   foreach item $rlist {
      set l [find-resource $type $item $resource]
      if {$l!={}} then {lappend ll $l}
   }
   return $ll
}

proc find-resource {type ps resource} {
   set l {}
   set role {}; set name {}; set host {}
   foreach p $ps {
      switch [first $p] {
         name {set name [second $p]}
         role {set role [second $p]}
         host {set host [second $p]}
         default {}
      }
   }
   if {$role==$resource}  then  {set l [set $type]}
   return $l
}

proc reset-current {} {
   global VXICrateList vxicrateindex CurrentVXIServer
   set CurrentVXIServer [lindex $VXICrateList $vxicrateindex]
   reset-VXI $CurrentVXIServer
}

proc reset-all {} {global VXIServer; reset-VXI $VXIServer}

proc setup-current {} {
   global VXICrateList vxicrateindex CurrentVXIServer
   set CurrentVXIServer [lindex $VXICrateList $vxicrateindex]
   setup-VXI $CurrentVXIServer
}

proc setup-all {} {global VXIServer; setup-VXI $VXIServer}

proc initialise-current {} {
   global VXICrateList vxicrateindex CurrentVXIServer
   set CurrentVXIServer [lindex $VXICrateList $vxicrateindex]
   initialise-VXI $CurrentVXIServer
}

proc initialise-all {} {global VXIServer; initialise-VXI $VXIServer}

proc initialise-VXI {server} {
   global VXIServer VXICrateList Trigpres VREpres

#    initialises a single crate or all crates dependent on value of parameter

    Stop-VXI $server

    if {"$server" == "$VXIServer"} then {
       foreach crate $VXICrateList {initialise-vxi-crate $crate}
    } else {
       initialise-vxi-crate $server
    }
}

proc initialise-vxi-crate {s} {
    global VXIModuleList CurrentVXIServer

    foreach module $VXIModuleList($CurrentVXIServer) {
        initialise-vxi-registers "$module.*"
    }
}

proc flush-current {} {
   global VXICrateList vxicrateindex CurrentVXIServer
   set CurrentVXIServer [lindex $VXICrateList $vxicrateindex]
   flush-VXI $CurrentVXIServer
}

proc flush-all {} {global VXIServer; flush-VXI $VXIServer}

proc flush-VXI {server} {
   global VXIServer VXICrateList Trigpres VREpres

#    flushes a single crate or all crates dependent on value of parameter

    Stop-VXI $server

    if {"$server" == "$VXIServer"} then {
       foreach crate $VXICrateList {flush-vxi-crate $crate}
    } else {
       flush-vxi-crate $server
    }

#  flush VRE firmware
 
   if {$VREpres} {
     set-footer "Flushing VREs"
     set-busy
     if {[inform vre vre-start] > 0} then {inform vre vre-flushDSP $server}
     clear-busy
     clear-footer
   }

   if {$Trigpres} then {return "VXI Flushed [date]"} \
     else {return "VXI Flushed -- Warning, no Trigger module present"}
}

proc flush-FIFO {name} {

#    flushes a single FIFO register

    set z [catch {write-vxi-register $name 0} m]
    if {$z != 0} then {
        midas-warning "Problems accessing $name - $m"
    }
    set z [catch {write-vxi-register $name 1} m]
    if {$z != 0} then {
        midas-warning "Problems accessing $name - $m"
    }
    set z [catch {write-vxi-register $name 0} m]
    if {$z != 0} then {
        midas-warning "Problems accessing $name - $m"
    }
}

proc enable-FIFO {name} {

#  reset and enable FIFO

    set z [catch {write-vxi-register $name 0} m]
    if {$z != 0} then {
        midas-warning "Problems accessing $name - $m"
    }
    set z [catch {read-vxi-register  $name} m]
    if {$z != 0} then {
        midas-warning "Problems accessing $name - $m"
    }
}


proc flush-vxi-crate {server} {
   global CurrentVXIServer  EGreport  VXIModuleList

#    flushes a single crate

    set CurrentVXIServer $server

    set-footer "Flushing crate $CurrentVXIServer"

    set EGreport_save $EGreport
    if {$EGreport == 0} then {set EGreport -1}

    foreach module $VXIModuleList($CurrentVXIServer) {
       switch -glob $module {

         Trig -
         BGO* -
         BgoBall* -
         Ge*  {
                if {[read-vxi-register $module.FIFO.NotEmpty] != 0} then {
                   write-vxi-register $module.FIFO.Flush 1
                   after 100
                   if {[read-vxi-register $module.FIFO.NotEmpty] != 0} then {
                      after 1000
                      if {[read-vxi-register $module.FIFO.NotEmpty] != 0} then {
                         midas-warning "Cannot empty $module FIFO"
                      }
                   }
                   write-vxi-register $module.FIFO.Flush 0
                }
                if {$module == "Trig"} then {
# flush event FIFO
                   flush-FIFO Trig.FIFO.Reset
                }
         }

         Clover* {
# flush FIFOs
                flush-FIFO $module.FIFO.Reset
          }

         CsI*   {
#  flush FIFOs
                flush-FIFO $module.FIFO.Reset
#  reset and enable GIR FIFOs
                enable-FIFO $module.GIR.FIFO.Test
         }

         Saphir*      -
         ClusterGe*  -
         ClusterBGO* {
#  flush FIFOs
                flush-FIFO $module.FIFO.Reset
#  reset and enable GIR FIFOs
                enable-FIFO $module.GIR.FIFO.Test
#  restart GIR DSP
#    Stop first to get ready for a Reset

                write-vxi-register $module.GIR.DSP.StopDSP 1

#    do the Reset
                write-vxi-register $module.GIR.DSP.ResetDSP 1

#    Start is now in go-vxi-crate
         }

         FVI* {
#  flush FIFOs
                flush-FIFO $module.FIFO.Reset
#  clear fera
                set mode [read-vxi-register $module.Chan1.FERACntrl]
                write-vxi-register $module.Chan1.FERACntrl 1
                write-vxi-register $module.Chan1.FERACntrl 0
                write-vxi-register $module.Chan1.FERACntrl $mode
                set mode [read-vxi-register $module.Chan2.FERACntrl]
                write-vxi-register $module.Chan2.FERACntrl 1
                write-vxi-register $module.Chan2.FERACntrl 0
                write-vxi-register $module.Chan2.FERACntrl $mode
#    start FVI 100 MHz clock
#    set test bit
                set mcr [read-vxi-register $module.MCR]
                write-vxi-register $module.MCR [expr ($mcr & 0xfff0) + 1]
#     rewrite original state without test bit
                write-vxi-register $module.MCR [expr ($mcr & 0xfffe)]
#    force the FVI to clear current event (and any inhibit)
#     set test + reject bits
                set mcr [read-vxi-register $module.MCR]
                write-vxi-register $module.MCR [expr ($mcr & 0xfff0) + 9]
#     rewrite original state without reject bit
                write-vxi-register $module.MCR [expr $mcr & 0xfff7]
# reset and enable GIR FIFOs
                enable-FIFO $module.GIR.FIFO.Test
         }

         TriggerMK2 -
         HRGe8x*   -
         ECC*      -
         ESS*      -
         GOCCE*    -
         TDR32V*    -
         TGIR*     -
         U2M*      -
         GMT*      -
         Centrum*  -
         *DC3214V* -
         *DC6414V* {
#    actions for these modules are loaded from the defn files
            set procname "_DAQ-reset"
            if {[string match TriggerMK2 $module]} then {set moduletype TriggerMK2}
            if {[string match QDC6414V* $module]} then {set moduletype QDC6414V}
            if {[string match ADC6414V* $module]} then {set moduletype ADC6414V}
            if {[string match QDC3214V* $module]} then {set moduletype QDC3214V}
            if {[string match ADC3214V* $module]} then {set moduletype ADC3214V}
            if {[string match TDC3214V* $module]} then {set moduletype TDC3214V}
            if {[string match U2M* $module]}      then {set moduletype U2M}
            if {[string match GMT* $module]}      then {set moduletype GMT}
            if {[string match HRGe8x* $module]}   then {set moduletype HRGe8x}
            if {[string match ECC* $module]}      then {set moduletype ECC}
            if {[string match ESS* $module]}      then {set moduletype ESS}
            if {[string match GOCCE* $module]}   then {set moduletype GOCCE}
            if {[string match TDR32V* $module]}   then {set moduletype TDR32V}
            if {[string match TGIR* $module]}    then {set moduletype TGIR}
            if {[string match Centrum* $module]}    then {set moduletype Centrum}
            global [set moduletype]_loaded
            if {![info exists [set moduletype]_loaded]} then {set [set moduletype]_loaded 0}
            if {[set [set moduletype]_loaded] == 0} then {
               set [set moduletype]_loaded [loadsource [string tolower $moduletype].defn]
            }
            if {[info procs $moduletype$procname] != ""} then {$moduletype$procname $module} 
         }
         default {}
      }
   }

   set EGreport $EGreport_save

   clear-footer
}

proc setup-vxi-crate {server} {
   global CurrentVXIServer  EGreport  VXIModuleList

#    perform any setup functions for a single crate

    set CurrentVXIServer $server

    set-footer "Setting up crate $CurrentVXIServer"

    set EGreport_save $EGreport
    if {$EGreport == 0} then {set EGreport -1}

    foreach module $VXIModuleList($CurrentVXIServer) {
       switch -glob $module {

         Trig -
         BGO* -
         BgoBall* -
         Ge*  { }

         Clover* { }

         CsI*   { }

         Saphir*      -
         ClusterGe*  -
         ClusterBGO* { }

         FVI* { }

         TriggerMK2 -
         HRGe8x*   -
         ECC*      -
         ESS*      -
         GOCCE*    -
         TDR32V*    -
         TGIR*     -
         U2M*      -
         GMT*      -
         Centrum*  -
         *DC3214V* -
         *DC6414V* {

#    actions for these modules are loaded from the defn files
            set procname "_DAQ-setup"
            if {[string match TriggerMK2 $module]} then {set moduletype TriggerMK2}
            if {[string match QDC6414V* $module]} then {set moduletype QDC6414V}
            if {[string match ADC6414V* $module]} then {set moduletype ADC6414V}
            if {[string match QDC3214V* $module]} then {set moduletype QDC3214V}
            if {[string match ADC3214V* $module]} then {set moduletype ADC3214V}
            if {[string match TDC3214V* $module]} then {set moduletype TDC3214V}
            if {[string match U2M* $module]}      then {set moduletype U2M}
            if {[string match GMT* $module]}      then {set moduletype GMT}
            if {[string match HRGe8x* $module]}   then {set moduletype HRGe8x}
            if {[string match ECC* $module]}      then {set moduletype ECC}
            if {[string match ESS* $module]}      then {set moduletype ESS}
            if {[string match GOCCE* $module]}    then {set moduletype GOCCE}
            if {[string match TDR32V* $module]}    then {set moduletype TDR32V}
            if {[string match TGIR* $module]}     then {set moduletype TGIR}
            if {[string match Centrum* $module]}    then {set moduletype Centrum}

            global [set moduletype]_loaded
            if {![info exists [set moduletype]_loaded]} then {set [set moduletype]_loaded 0}
            if {[set [set moduletype]_loaded] == 0} then {
               set [set moduletype]_loaded [loadsource [string tolower $moduletype].defn]
            }
            if {[info procs $moduletype$procname] != ""} then {$moduletype$procname $module} 
         }
         default {}
      }
   }

   set EGreport $EGreport_save

   clear-footer
}


proc go-vxi-crate {server} {
   global CurrentVXIServer  EGreport  VXIModuleList

#    GOes a single crate

    set CurrentVXIServer $server

    set-footer "Going crate $CurrentVXIServer"

    set EGreport_save $EGreport
    if {$EGreport == 0} then {set EGreport -1}

    foreach module $VXIModuleList($CurrentVXIServer) {
       switch -glob $module {

         Trig {
# Trig event number must be set in test mode
            write-vxi-register Trig.TestMode 1
            write-vxi-register Trig.EventNumber 0
            write-vxi-register Trig.TestMode 0
         }

         BGO* -
         BgoBall* -
         Ge*  {
         }

         Clover* {
#  zero event numbers
             write-vxi-register $module.RdOut.EvNum.Count 0
         }

         FVI* {
#  zero GIR event numbers
#             write-vxi-register $module.GIR.EvNum.Count 0     removed 5/11/99  
         }

         CsI*  {
#  zero GIR event numbers
             write-vxi-register $module.GIR.EvNum.Count 0
         }

         Saphir* -
         ClusterGe*  -
         ClusterBGO* {
#  zero GIR event numbers
             write-vxi-register $module.GIR.EvNum.Count 0
#    start the GIR DSP
             write-vxi-register $module.GIR.DSP.StartDSP 1
         }

         TriggerMK2 -
         CsI*      -
         HRGe8x*   -
         ECC*      -
         ESS*      -
         GOCCE*    -
         TDR32V*    -
         TGIR*     -
         U2M*      -
         GMT*      -
         Centrum*  -
         *DC3214V* -
         *DC6414V* {
#    actions for these modules are loaded from the defn files
            set procname "_DAQ-go"
            if {[string match TriggerMK2 $module]} then {set moduletype TriggerMK2}
            if {[string match QDC6414V* $module]} then {set moduletype QDC6414V}
            if {[string match ADC6414V* $module]} then {set moduletype ADC6414V}
            if {[string match QDC3214V* $module]} then {set moduletype QDC3214V}
            if {[string match ADC3214V* $module]} then {set moduletype ADC3214V}
            if {[string match TDC3214V* $module]} then {set moduletype TDC3214V}
            if {[string match U2M* $module]}      then {set moduletype U2M}
            if {[string match GMT* $module]}      then {set moduletype GMT}
            if {[string match HRGe8x* $module]}   then {set moduletype HRGe8x}
            if {[string match ECC* $module]}      then {set moduletype ECC}
            if {[string match ESS* $module]}      then {set moduletype ESS}
            if {[string match GOCCE* $module]}    then {set moduletype GOCCE}
            if {[string match TDR32V* $module]}    then {set moduletype TDR32V}
            if {[string match TGIR* $module]}     then {set moduletype TGIR}
            if {[string match Centrum* $module]}    then {set moduletype Centrum}

            global [set moduletype]_loaded
            if {![info exists [set moduletype]_loaded]} then {set [set moduletype]_loaded 0}
            if {[set [set moduletype]_loaded] == 0} then {
               set [set moduletype]_loaded [loadsource [string tolower $moduletype].defn]
            }
            if {[info procs $moduletype$procname] != ""} then {$moduletype$procname $module} 
         }

         default {}
      }
   }

   set EGreport $EGreport_save

   clear-footer
}

proc halt-vxi-crate {server} {
   global CurrentVXIServer  EGreport  VXIModuleList

#    HALTs a single crate

    set CurrentVXIServer $server

    set-footer "Halting crate $CurrentVXIServer"

    set EGreport_save $EGreport
    if {$EGreport == 0} then {set EGreport -1}

    foreach module $VXIModuleList($CurrentVXIServer) {
       switch -glob $module {

         Trig {
         }

         BGO* -
         BgoBall* -
         Ge*  {
         }

         Clover* {
         }

         FVI* -
         Saphir*   -
         CsI*      -
         ClusterGe*  -
         ClusterBGO* {
         }

         TriggerMK2 -
         HRGe8x*   -
         ECC*      -
         ESS*      -
         GOCCE*    -
         TDR32V*    -
         TGIR*     -
         U2M*      -
         GMT*      -
         Centrum*  -
         *DC3214V* -
         *DC6414V* {
#    actions for these modules are loaded from the defn files
            set procname "_DAQ-halt"
            if {[string match TriggerMK2 $module]} then {set moduletype TriggerMK2}
            if {[string match QDC6414V* $module]} then {set moduletype QDC6414V}
            if {[string match ADC6414V* $module]} then {set moduletype ADC6414V}
            if {[string match QDC3214V* $module]} then {set moduletype QDC3214V}
            if {[string match ADC3214V* $module]} then {set moduletype ADC3214V}
            if {[string match TDC3214V* $module]} then {set moduletype TDC3214V}
            if {[string match U2M* $module]}      then {set moduletype U2M}
            if {[string match GMT* $module]}      then {set moduletype GMT}
            if {[string match HRGe8x* $module]}   then {set moduletype HRGe8x}
            if {[string match ECC* $module]}      then {set moduletype ECC}
            if {[string match ESS* $module]}      then {set moduletype ESS}
            if {[string match GOCCE* $module]}    then {set moduletype GOCCE}
            if {[string match TDR32V* $module]}    then {set moduletype TDR32V}
            if {[string match TGIR* $module]}     then {set moduletype TGIR}
            if {[string match Centrum* $module]}    then {set moduletype Centrum}

            global [set moduletype]_loaded
            if {![info exists [set moduletype]_loaded]} then {set [set moduletype]_loaded 0}
            if {[set [set moduletype]_loaded] == 0} then {
               set [set moduletype]_loaded [loadsource [string tolower $moduletype].defn]
            }
            if {[info procs $moduletype$procname] != ""} then {$moduletype$procname $module} 
         }

         default {}
      }
   }

   set EGreport $EGreport_save

   clear-footer
}

proc reset-VXI {server} {
   global VXIServer VXICrateList Trigpres VREpres TrigModule CurrentVXIServer

#    resets a single crate or all crates dependent on value of parameter

    if {"$server" != "$VXIServer"} then {Redefineregisters}

    Stop-VXI $server

#    flush-VXI $server

    if {"$server" == "$VXIServer"} then {
       foreach crate $VXICrateList {flush-vxi-crate $crate}
    } else {
       flush-vxi-crate $server
    }

# Trigger
    if {$Trigpres != 0 && $TrigModule == "Trig"} then {
#    Eurogam trigger used with Euroball is always in readout test mode
       write-vxi-register $TrigModule.ReadoutMode 1
    }

#  load VRE firmware

   if {$VREpres} {
     set-footer "RESETting VREs"
     set-busy
     if {[inform vre vre-start] > 0} then {inform vre vre-loadDSP $server}
     clear-busy
     clear-footer
   }

#    set flag indicating Reset complete
#    if the Reset is for all crates this will set the flag in all
#    crates but we only get here if all crates have completed

   set CurrentVXIServer $server
   write-vxi-registers NetVar_HasBeenReset 1

   if {$Trigpres} then {return "VXI Reset [date]"} \
     else {return "VXI reset  --  Warning no Trigger module present"}
}

proc setup-VXI {server} {
   global VXIServer VXICrateList Trigpres TrigModule CurrentVXIServer

#    setup a single crate or all crates dependent on value of parameter

    if {"$server" != "$VXIServer"} then {Redefineregisters}

    Stop-VXI $server

#    setup-VXI $server

    if {"$server" == "$VXIServer"} then {
       foreach crate $VXICrateList {setup-vxi-crate $crate}
    } else {
       setup-vxi-crate $server
    }

#    set flag indicating Setup complete
#    if the Setup is for all crates this will set the flag in all
#    crates but we only get here if all crates have completed

   set CurrentVXIServer $server
   write-vxi-registers NetVar_HasBeenSetup 1
}

proc Stop-VXI {server} {
  global Trigpres TrigModule VXIServer VXICrateList VXIModuleList CurrentVXIServer

    if {$Trigpres} then {
#    stop all crates via Master Trigger
       write-vxi-register $TrigModule.VXI.Go 0
    } else {
#    no Master Trigger so stop VXI via crate RM or RMs dependent on server parameter
#    if no RM present look for a SlotOne module
       if {"$server" == "$VXIServer"} then {
          foreach crate $VXICrateList {
             set CurrentVXIServer $crate
             set RM [lindex $VXIModuleList($crate) [lsearch $VXIModuleList($crate) RM*]]
             if {"$RM" == ""} then {
                 set RM [lindex $VXIModuleList($crate) [lsearch $VXIModuleList($crate) SlotOne*]]
             }
             if {"$RM" != ""} then {write-vxi-register $RM.VXI.Go 0}
          }
       } else {
          set crate $server
          set CurrentVXIServer $crate
          set RM [lindex $VXIModuleList($crate) [lsearch $VXIModuleList($crate) RM*]]
          if {"$RM" == ""} then {
              set RM [lindex $VXIModuleList($crate) [lsearch $VXIModuleList($crate) SlotOne*]]
          }
          if {"$RM" != ""} then {write-vxi-register $RM.VXI.Go 0}
       }
    }
}

proc Go-VXI {server} {
  global Trigpres TrigModule VXIServer VXICrateList VXIModuleList CurrentVXIServer

    if {$Trigpres} then {
#    go all crates via Master Trigger
       write-vxi-register $TrigModule.VXI.Go 1
    } else {
#    no Master Trigger so go VXI via crate RM or RMs dependent on server parameter
#    if no RM present look for a SlotOne module
       if {"$server" == "$VXIServer"} then {
          foreach crate $VXICrateList {
             set CurrentVXIServer $crate
             set RM [lindex $VXIModuleList($crate) [lsearch $VXIModuleList($crate) RM*]]
             if {"$RM" == ""} then {
                 set RM [lindex $VXIModuleList($crate) [lsearch $VXIModuleList($crate) SlotOne*]]
             }
             if {"$RM" != ""} then {write-vxi-register $RM.VXI.Go 1}          }
       } else {
          set crate $server
          set CurrentVXIServer $server
          set RM [lindex $VXIModuleList($crate) [lsearch $VXIModuleList($crate) RM*]]
          if {"$RM" == ""} then {
              set RM [lindex $VXIModuleList($crate) [lsearch $VXIModuleList($crate) SlotOne*]]
          }
          if {"$RM" != ""} then {write-vxi-register $RM.VXI.Go 1}
       }
    }
}

proc read-vxistate {} {
  global Trigpres TrigModule CurrentVXIServer VXICrateList VXIModuleList

#    proc called by sys task
#    note - if there is no Master Trigger in the system will use first RM

   if {$Trigpres} then {
      set state [read-vxi-register $TrigModule.VXI.Go]
   } else {
      set crate [first $VXICrateList]
      set CurrentVXIServer $crate
      set RM [lindex $VXIModuleList($crate) [lsearch $VXIModuleList($crate) RM*]]
      if {"$RM" == ""} then {
          set RM [lindex $VXIModuleList($crate) [lsearch $VXIModuleList($crate) SlotOne*]]
      }
      if {"$RM" == ""} then {set state 0} else {set state [read-vxi-register $RM.VXI.Go]}
   }
   return $state
}

proc format-data l {
  set ll {}
  foreach f $l {lappend ll [format "%20s %16s %10s %10s" \
   [first $f] [second $f] [third $f] [fourth $f] ]}
  return $ll
}

proc make-datalist {} {
   global modulechoice confignames
   global datalist

#    delete the table occupied and recreate as a way of deleting all elements
   global occupied; catch {unset occupied}; global occupied
   set selection 0
   foreach data $datalist {
      set moduletype "[first $data]"
      set modulenumber [third $data]
      set vxislot [fourth $data]

      set moduleindex [lsearch $modulechoice $moduletype]
      set modulename [lindex $confignames $moduleindex]

      if {[info exists occupied(name,n$vxislot)] == 0} then {
#      first use of vxi crate slot
         set occupied(name,n$vxislot) [lindex $confignames $moduleindex]
      } else {
#      vxi crate slot already mentioned
         display-error-report self  \
   "VXI slot $vxislot used by  [set occupied(name,n$vxislot)] and [lindex $confignames $moduleindex]"
      }

      set datalist [lreplace $datalist $selection $selection \
         [list $moduletype $modulename $modulenumber $vxislot ]]
      incr selection
   }
   paint-datalist
   repaint
}

proc load-data n {
   global datalist moduletype modulename vxislot modulenumber
   set data [lindex $datalist $n]
   set moduletype [first $data]
   set modulename [second $data]
   set modulenumber [third $data]
   set vxislot [fourth $data]
}

proc paint-datalist {} {
  global datalist  selection
  global m
  set pointstrings [format-data $datalist]
  set strings [format-data $m]
    :modify M :empty T :strings "$strings"
    :modify CONFIG-DATALIST :show F :empty T :strings "$pointstrings" :show T
    :modify CONFIG-DATALIST :select $selection
}

proc repaint {} {
   global debug moduletype modulename vxislot modulenumber
   global modulechoice slotchoice
   if {$debug} then {puts stderr "repaint"}
   set mt [lsearch $modulechoice $moduletype]
   set cn [lsearch $slotchoice $vxislot]
     :modify MODULE :value $mt
     :modify SLOT :value $cn

  global [set modulename]choice
  set lmodulenumber [lsearch [set [set modulename]choice] $modulenumber]
  :modify NUMBER :strings [set [set modulename]choice] :value $lmodulenumber
}

proc click-add {} {
   global datalist moduletype modulename vxislot modulenumber
   lappend datalist [list $moduletype $modulename $modulenumber $vxislot ]
   make-datalist
   clear-footer
   }

proc click-replace {} {
   global selection datalist moduletype modulename vxislot modulenumber
   if {$selection<0} then \
      {set-footer "you haven't selected a line to replace!"; return}
   set datalist [lreplace $datalist $selection $selection \
                     [list $moduletype $modulename $modulenumber $vxislot]]
   make-datalist
   clear-footer
   }

proc click-delete {} {
   global selection datalist
   if {$selection<0} then \
      {set-footer "you haven't selected a line to delete!"; return}
   set datalist [lreplace $datalist $selection $selection]
   set selection -1
   make-datalist
   clear-footer
   }

proc click-clear {} {
   global datalist
   set datalist {}
   paint-datalist
}

proc click-datalist {c s} {
  global debug selection
  if {$debug} then {puts stderr "click-datalist with arguments \{$c $s\}"}
  case $c in {
    :list-select {set selection $s; load-data $selection}
    :list-deselect {set selection -1}
  }
  repaint
  clear-footer
}

proc double-click-datalist {c s} {
  global debug selection moduletype tasknames modulechoice
  if {$debug} then {puts stderr "double-click-datalist with arguments \{$c $s\}"}
  set selection $s; load-data $selection
  set taskname [lindex $tasknames [lsearch $modulechoice $moduletype]]
  inform $taskname start
}

proc click-module {n} {
  global debug modulechoice confignames moduletype modulename modulenumber

  if {$debug} then {puts stderr "click-module with argument \{$n\}"}
  set moduletype [lindex $modulechoice $n]
  set modulename [lindex $confignames $n]
  global [set modulename]choice
  set modulenumber [first [set [set modulename]choice]]
  :modify NUMBER :strings [set [set modulename]choice] :value 0
}

proc click-crate {n} {
  global VXICrateList VXIHostList VXIModuleList vxicrate vxihost vximodules vxicrateindex
  global configdirectory vxihost
  set vxicrateindex $n
  set vxicrate [lindex $VXICrateList $vxicrateindex]
  set vxihost  [lindex $VXIHostList $vxicrateindex]
  set vximodules $VXIModuleList($vxicrate)
  :modify CRATE :value $vxicrateindex
  :modify HOST  :value $vxihost
  :modify CONFIG :value [file join  $configdirectory $vxihost]
  click-load
}

proc select-crate {crate} {
    global VXICrateList
    click-crate [lsearch $VXICrateList $crate]
}

proc click-slot {n} {
  global slotchoice vxislot
  set vxislot [lindex $slotchoice $n]
}

proc click-modulenumber {n} {
  global  modulenumber modulename
  global [set modulename]choice
  set modulenumber [lindex [set [set modulename]choice] $n]
}

proc click-load {} {
  global configdirectory vxihost datalist
  global modulenames confignames
  global modulename moduletype modulechoice modulenumber vxislot

  set file [file join  $configdirectory $vxihost]
  set f [open $file r]
  set l [split [read $f] \n]
  close $f

  set datalist {}

  foreach item $l {
    if {[first $item] == "module"} then {
       set m [second $item]
       set modulename [string trimright $m #0123456789]
       if {$modulename == "TriggerMK"} then {set modulename "TriggerMK2"}
       set moduletype [lindex $modulechoice [lsearch $confignames $modulename]]
       set modulenumber [string range $m [string length $modulename] end]
       if {[string range $modulenumber 0 0] == "#"} then {set modulenumber [string range $modulenumber 1 end]}
       set vxislot [fourth $item]
       lappend datalist [list $moduletype $modulename $modulenumber $vxislot ]
    }
  }
  make-datalist
  set-footer "Crate configuration loaded from file $file"
}

proc click-save {} {
  global configdirectory vxihost datalist

  set file [file join  $configdirectory $vxihost]
  set z [catch {open $file w} f]
  if {$z != 0} then {set-footer $f; return}

   foreach data $datalist {
      set modulename [second $data]
      set modulenumber      [third $data]
      set vxislot    [fourth $data]

      if {$modulename == "Icare" || $modulename == "Demon" || $modulename == "Saphir" || $modulename == "CsI" || \
           $modulename == "HRGe8x" || $modulename == "ECC" || $modulename == "ESS" || $modulename == "BgoBall"} then {
         puts $f "module [set modulename]#[set modulenumber]  slot $vxislot"
      } else {
         puts $f "module $modulename$modulenumber  slot $vxislot"
      }
      if {$modulename == "Ge" || $modulename == "BGO"} then {
          foreach c {1 2 3 4 5 6} {
             puts $f "    detector $modulename[expr [expr [expr $modulenumber * 6] + $c] -1] channel $c"
          }
      }
      if {$modulename == "Saphir"} then {
          foreach c {1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16} {
             puts $f "    detector $modulename[expr [expr [expr $modulenumber * 16] + $c] -1] channel $c"
          }
      }
      if {$modulename == "CsI"} then {
          foreach c {1 2 3 4 5 6 7 8} {
             puts $f "    detector $modulename[expr [expr [expr $modulenumber * 8] + $c] -1] channel $c"
          }
      }
      if {$modulename == "HRGe8x" || $modulename == "ECC"} then {
          foreach c {1 2} {
             puts $f "    detector $modulename[expr [expr [expr $modulenumber * 2] + $c] -1] channel $c"
          }
      }
      if {$modulename == "ESS"} then {
          foreach c {1 2 3 4} {
             puts $f "    detector $modulename[expr [expr [expr $modulenumber * 4] + $c] -1] channel $c"
          }
      }
      if {$modulename == "Icare"} then {
          foreach c {1 2 3 4 5 6} {
             puts $f "    detector $modulename[expr [expr $modulenumber * 10] + $c] channel $c"
          }
      }
      if {$modulename == "Demon"} then {
          foreach c {1 2 3 4 5 6} {
             puts $f "    detector $modulename[expr [expr $modulenumber * 6] + $c] channel $c"
          }
      }
      if {$modulename == "BgoBall"} then {
          foreach c {1 2 3 4} {
             puts $f "    detector $modulename[expr [expr [expr $modulenumber * 4] + $c] -1] channel $c"
          }
      }
   }
   close $f
   set-footer "Crate configuration saved to file $file"

   Make_Configuration_File 
}

proc Make_Configuration_File {} {}

proc Reconfigure {} {
  global CurrentVXIServer vxicrate
#    re-configure a VXI crate

   if {![midas-confirmation " VXI Crate ReConfiguration\n\
        This will destroy all unsaved setups in the VXI crate\n\
        Please confirm that you wish to proceed"]} then {return}

   set CurrentVXIServer $vxicrate

   set file [read-vxi-register config]
   :modify CONFIG :value $file

   set-footer "Reconfiguring the Crate..."
   EG "register :name '/$CurrentVXIServer/config' !initialise"
   set-footer "Crate reconfigured at [date] using file $file"

   Redefineregisters
   write-vxi-register NetVar_HasBeenRestored 0
   write-vxi-register NetVar_HasBeenReset 0

   configure-current
   reset-VXI $CurrentVXIServer

   if {[midas-confirmation  " Crate re-configuration complete\n\
     If you have moved any VXI modules between crates or if you have\
     moved a Master Trigger module then you must use the \"Check All Crates\" function.\n\
     Perform \"Check All Crates\" now?"]} then {configure-all 0}

   if {[midas-confirmation " The VXI crate re-configuration requires that you\
        restore the setup of all modules in the VXI crate\n\
        Please confirm that you wish to proceed"]} then {inform vxirestore start}
}

proc ShowConfiguration {} {
   global configdirectory vxihost
   midas-show-file [file join  $configdirectory $vxihost]
}

proc Redefineregisters {} {
   global configdirectory NetVarRegDefn
#    after a crate reconfiguration some registers will be lost
   if {[ersinquire "NetVar_*"] == ""} then {source [file join  $configdirectory $NetVarRegDefn]}
}

set VXIconfigured 0

proc configure-current {} {
   global vxicrate vximodules VXIModuleList
   VXI-Configuration $vxicrate
   set vximodules $VXIModuleList($vxicrate)
   update-configuration
}

proc configure-crate {crate} {
   select-crate $crate
   configure-current 
}

proc configure-all {mode} {
   global VXICrateList VXIModuleList VXIconfigured
   global vxicrate vximodules vxicrateindex

   set-footer "Please wait a few moments - checking VXI configuration"
   catch {unset VXIModuleList}
   catch {unset VXICrateList}
   make-cratelist

   set vxicrateindex_save $vxicrateindex
   set index 0
   foreach crate $VXICrateList {
      if {$mode == 0} then {
         click-crate $index
      } else {
         set vxicrate $crate
      }
      VXI-Configuration $vxicrate
      set vximodules $VXIModuleList($vxicrate)
      if {$mode == 0} then {
         update-configuration
         update
         after 1000
      }
      incr index
   }
   click-crate $vxicrateindex_save
   set vximodules $VXIModuleList($vxicrate)
   if {$mode == 0} then {update-configuration}

   set VXIconfigured 1
   Find_Trigger
   set-footer "Ready"
}

proc VXI-Configuration {CrateList} {
   global VXIModuleList VXICrateList datalist
#    Update the map lists with the current VXI configuration
   foreach crate $CrateList {
#    obtain result of module VXI self-tests
      set server [VXIcrate2server $crate]
      set-footer "Probing VXI crate $crate server $server"
      insert-log "Probing VXI crate $crate server $server"
      click-crate [lsearch $VXICrateList $crate]
      set modulelist {}
      foreach item $datalist {
         set mname [second $item]
         set mnum  [third $item]
         set chanspermodule 1
         if {$mname == "Ge" || $mname == "BGO"} then {set chanspermodule 6}
         if {$mname == "Saphir"} then {set chanspermodule 16}
         if {$mname == "CsI"} then {set chanspermodule 8}
         if {$mname == "HRGe8x" || $mname == "ECC"} then {set chanspermodule 2}
         if {$mname == "ESS"} then {set chanspermodule 4}
         if {$mname == "Icare"} then {set chanspermodule 6}
         if {$mname == "Demon"} then {set chanspermodule 6}
         if {$mname == "BgoBall"} then {set chanspermodule 4}
         if {$mname == "TriggerMK2" || $mname == "Trig"} then {
             set chanspermodule 0
             lappend modulelist $mname
         }
         for {set c 0} {$c < $chanspermodule} {incr c} {
             set i [expr [third $item] * $chanspermodule + $c]
             if {$mname == "Icare"} then {set i [expr [third $item] * 10 + $c + 1]}
             if {$mname == "Demon"} then {set i [expr [third $item] * 6 + $c + 1]}
             set module [set mname]$i
             lappend modulelist $module
         }
      }
#    finally add the crate list to a list for all crates
      set VXIModuleList($crate) [lsort $modulelist]
      insert-log "crate $crate contains $VXIModuleList($crate)"
   }
   clear-footer
}

proc VXI-state {} {
   global VXICrateList VXIModuleList EGreport Trigpres TrigModule

#    create a window for the status information
    set frame [self frame-name].status

    :frame $frame :label  "VXI System status at [date]" :transient T
    :galley G :deaf T :strings ""
    :show T

    set EGreport_save $EGreport
    if {$EGreport == 0} then {set EGreport -1}

    if {$Trigpres} then {
      :modify G :strings "System contains a Master Trigger module "
      set state [read-vxi-register $TrigModule.VXI.Go]
      if {$state == 0} then {set s NOT} else {set s ""}
      :modify G :strings "and its VXI GO register is $s set"
    } else {
      :modify G :strings "System does NOT contain a Master Trigger module"
    }

   foreach crate $VXICrateList {
#  obtain status of this VXI crate
      set server [VXIcrate2server $crate]
      :modify G :strings "\n\nVXI crate $crate has server $server\n"
      set s [EG "register :name '/$server/NetVar_MyHostName' ?value"]
      :modify G :strings "Server has host name $s\n"
      set s [EG "register :name '/$server/NetVar_MyVXICrate' ?value"]
      :modify G :strings "Server has name $s\n"
      set s [EG "register :name '/$server/NetVar_ServerType' ?value"]
      :modify G :strings "Server has type $s\n"
      set s [EG "register :name '/$server/NetVar_ServerOS' ?value"]
      :modify G :strings "Server has OS $s\n"
      set state [EG "register :name '/$server/NetVar_HasBeenReset' ?value"]
      if {$state == 0} then {set s NOT} else {set s ""}
      :modify G :strings "Crate has $s been software reset\n"
      set state [EG "register :name '/$server/NetVar_HasBeenSetup' ?value"]
      if {$state == 0} then {set s NOT} else {set s ""}
      :modify G :strings "Crate has $s been software setup\n"
      set state [EG "register :name '/$server/NetVar_HasBeenRestored' ?value"]
      if {$state == 0} then {set s NOT} else {set s ""}
      :modify G :strings "A parameter restore has $s been performed\n"
      set RM [lindex $VXIModuleList($crate) [lsearch $VXIModuleList($crate) RM*]]
      if {"$RM" == ""} then {
          set RM [lindex $VXIModuleList($crate) [lsearch $VXIModuleList($crate) SlotOne*]]
      }
      if {"$RM" == ""} then {set RMstate 0} else {set RMstate [EG "register :name '/$server/$RM.TTLTRG' ?value"]}
      set state [expr $RMstate >> 4 & 1]
      if {$state == 0} then {set s NOT} else {set s ""}
      :modify G :strings "VXI GO backplane line is $s set\n"
      if {"$RM" == ""} then {set RMstate 0} else {set RMstate [EG "register :name '/$server/$RM.VXI.Go' ?value"]}
      if {$state == 0} then {set s NOT} else {set s ""}
      :modify G :strings "RM VXI GO register is $s set\n"
      set state [EG "register :name '/$server/NetVar_Inspection_AI1' ?value"]
      :modify G :strings "Inspection line analogue 1 is $state\n"
      set state [EG "register :name '/$server/NetVar_Inspection_AI2' ?value"]
      :modify G :strings "Inspection line analogue 2 is $state\n"
      set state [EG "register :name '/$server/NetVar_Inspection_DI1' ?value"]
      :modify G :strings "Inspection line digital 1  is $state\n"
      set state [EG "register :name '/$server/NetVar_Inspection_DI2' ?value"]
      :modify G :strings "Inspection line digital 2  is $state\n"
      set state [EG "register :name '/$server/NetVar_Inspection_VI' ?value"]
      :modify G :strings "Inspection line voltage    is $state\n"
   }
   set EGreport $EGreport_save
}

proc update-configuration {} {
  global datalist  selection
  set pointstrings [format-configuration $datalist]
  :modify CONFIG-DATALIST :show F :empty T :strings "$pointstrings" :show T
  :modify CONFIG-DATALIST :select $selection
}

proc format-configuration l {
  global vximodules Unknown CurrentVXIServer vxicrate

  set CurrentVXIServer $vxicrate

  set ll {}

  foreach f $l {
     set mname [second $f]
     set chanspermodule 1
     if {$mname == "Ge" || $mname == "BGO"} then {set chanspermodule 6}
     if {$mname == "Saphir"} then {set chanspermodule 16}
     if {$mname == "CsI"} then {set chanspermodule 8}
     if {$mname == "HRGe8x" || $mname == "ECC"} then {set chanspermodule 2}
     if {$mname == "ESS"} then {set chanspermodule 4}
     if {$mname == "Icare"} then {set chanspermodule 6}
     if {$mname == "Demon"} then {set chanspermodule 6}
     if {$mname == "BgoBall"} then {set chanspermodule 4}
     if {[third $f] == ""} then {
         set tstnumber ""
     } else {
         set tstnumber [expr [third $f] * $chanspermodule]
         if {$mname == "Icare"} then {set tstnumber [expr [third $f] * 10 + 1]}
         if {$mname == "Demon"} then {set tstnumber [expr [third $f] * 6 + 1]}
     }
     if {[lsearch $vximodules $mname$tstnumber] == -1} then {
         set s $Unknown
     } else {
        set r [read-vxi-registers $mname$tstnumber.VXI.Passed]
        if {"$r" == ""} then {
            set s "Unknown"
        } else {
            if {[second [first $r]] == 1} then {set s "Passed"} else {set s "Present"}
        }
     }
     
     lappend ll [format "%20s %16s %10s %10s %8s" \
       [first $f] [second $f] [third $f] [fourth $f] $s ]
  }
  return $ll
}

proc VXImodule2crate {requested} {
  global VXIconfigured Unknown
  global VXICrateList VXIModuleList VXIHostList
    if {$VXIconfigured == 0} then {configure-all 1}
    set index 0
    foreach crate $VXICrateList {
       set modulelist $VXIModuleList($crate)
       if {[lsearch $modulelist $requested] != -1} then {
#          found the crate we are looking for
#          return server name to access this crate
         return [VXIcrate2server $crate]
       }
       incr index
    }
#    update VXI configuration and try again
#    configure-all
#    foreach crate $VXICrateList {
#      set modulelist $VXIModuleList($crate)
#      if {[lsearch $modulelist $requested] != -1} then {return $crate}
#    }
#    still no luck - give up
   return $Unknown
}

proc VXIcrate2server {requested} {
  global VXICrateList VXIHostList
  set host [lindex $VXIHostList [lsearch $VXICrateList $requested]]
  return $requested
}

proc double-click arg {
   global debug
   if {$debug} then {puts stderr "double-click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   case $w in {

        CONFIG-DATALIST {double-click-datalist $c [lindex $arg 4]}

      default   {puts stdout $arg}
   }
}

proc do_vxiactions {w} {

   if {[midas-confirmation "$w\n\
        Please confirm that you wish to proceed"]} then {[string tolower $w]-current}
}

proc click arg {
   global debug env
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]
   if {$c==":quit"} then {click-quit; return}
   switch $w  {

        MODULE  {click-module $v}
        SLOT    {click-slot $v}
        NUMBER  {click-modulenumber $v}

        ADD     {click-add}
        REPLACE {click-replace}
        DELETE  {click-delete}
        CLEAR   {click-clear}
        SAVE    {click-save}
        LOAD    {click-load}

        VXIACTIONS {do_vxiactions $v}

        CONFIG-DATALIST {click-datalist $c [lindex $arg 4]}
        CRATE   {click-crate $v}

        default   {puts stdout $arg}
    }
}

#    called by ex when experiment joined
proc set-experiment args {
   global env VXIconfigured NULL started
   set VXIconfigured 0
   if {$env(EXPERIMENTBASE) == "" || $env(EXPERIMENTBASE) == "$NULL"} then {return}
   if {$started == 0} then {start; close-frame} else {start}
}

proc autostart args {
   global env started NULL
   if {$env(EXPERIMENTBASE) == "" || $env(EXPERIMENTBASE) == "$NULL"} then {return}
   if {$started == 0} then {start; close-frame} else {continue}
}

proc start args {
   global started VXIconfigured
   if {$started == 0} then {make-frame}
   open-frame
   if {$VXIconfigured != 0} then {return}
   make-cratelist
   Find_Trigger
}


proc click-quit args {close-frame}

proc finish args {destroy-frame}

set slotchoice {0 1 2 3 4 5 6 7 8 9 10 11 12}
set RMchoice {1 2 3 4 5 6 7 8 9 10}
set ROCOchoice {1 2 3 4 47}
set VREchoice {1 2 3 4 5 6 7 15  17 18 19 31  33 34 35 47  49 50 51 63 127}
set Trigchoice {""}
set TriggerMK2choice {""}
set FVIchoice {1 2}
set Gechoice     {0 1 2 3 4 5 30 31}
set BGOchoice    {0 1 2 3 4 5 6 7 8 9 10}
set BgoBallchoice    {0 1 2 3 4 5}
set Cloverchoice {0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20  \
                  21 22 23 24 25 26}
set TestCardchoice {0 1 2 3}
set ClusterGechoice  {0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15}
set ClusterBGOchoice {0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15}
set Saphirchoice {0 1 2 3 4 5 6 7}
set HRGe8xchoice {0 1 2 3 4 5 6 7}
set ECCchoice {0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15}
set ESSchoice {0 1 2 3 4 5 6 7}
set TDR32Vchoice {0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15}
set SlotOnechoice {0 1 2 3}
set Centrumchoice {0}
set GOCCEchoice {0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15}
set TGIRchoice {0 1 2 3 4 5 6 7}
set CsIchoice    {0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15}
set Icarechoice {0 1 2 3 4 5 6 7 8 9}
set Demonchoice {0 1 2 3 4 5 6 7 8 9}
set QDC6412Vchoice {0 1 2 3 4 5 6 7}
set QDC6414Vchoice {0 1 2 3 4 5 6 7}
set QDC3214Vchoice {0 1 2 3 4 5 6 7}
set ADC6414Vchoice {0 1 2 3 4 5 6 7}
set ADC3214Vchoice {0 1 2 3 4 5 6 7}
set TDC3214Vchoice {0 1 2 3 4 5 6 7}
set U2Mchoice {0}
set GMTchoice {0}


proc make-frame {} {
   global env started
   global modulechoice slotchoice VXICrateList

   set started 1

  :frame [self frame-name] :label "[self frame-title]"     \
      :show-footer T  :resize F \
      :icon-image "@$env(ICONHOME)/vxi_crate" \
      :icon-name "[self icon-title]"

   :panel CONFIGURATION  :layout T :padx 10 :pady 10
      :layout H; :row-gap 10; :col-gap 10
      :choice-stack CRATE :label "VXI Crate Name:" :strings $VXICrateList
      :hskip 32
      :text HOST :label "Host Name" :w 8 :deaf T
      :text CONFIG  :label "Configuration File" :w 24 :deaf T
      :next-row T
      :layout V
      :list M :rows 2 :cols 80 :strings [list ""] :scroll N :deaf T
      :list CONFIG-DATALIST :rows 14 :cols 80 :exclusive T \
        :strings [list ""] :double-click T
   :panel OPTIONS  :layout T :padx 10
      :layout H; :row-gap 10; :col-gap 10
      :choice-stack MODULE  :label "Module Type:"   :exclusive T  :strings "$modulechoice" :value 0
      :hskip 70
      :choice-stack SLOT :label "VXI Slot:" :exclusive T  :strings "$slotchoice" :value 0
      :choice-stack NUMBER :label "Module Number:"  :exclusive T  :strings ""  :value 1

   :panel CLICK :layout T :padx 10
      :layout H; :row-gap 5; :col-gap 5
      :vskip 10
      :button ADD     :label Add
      :button REPLACE :label Replace
      :button DELETE  :label Delete
      :button CLEAR   :label Clear
      :button LOAD    :label "Load"
      :button SAVE    :label "Save"
      :hskip 20
      :button RECONFIG        :label "Reconfigure Crate" :command Reconfigure
      :button SHOWFILE        :label "Show file"  :command ShowConfiguration
      :next-row T
      :col-gap 10
      :button RECONFIGURE     :label "Check Crate"       :command configure-current
      :button RECONFIGUREALL  :label "Check All Crates"  :command "configure-all 0"
      :menubutton VXIACTIONS  :label "Act on Crate"      :strings [list Reset Flush Setup Initialise]
      :next-row T
      :col-gap 5
      :button HALTVXI    :label "HALT VXI"  :command DAQ-halt
      :button GOVXI      :label "GO VXI"    :command DAQ-go
      :button FLUSHVXI   :label "FLUSH VXI" :command DAQ-flush
      :button RESETVXI   :label "RESET VXI" :command DAQ-reset
      :button SETUPVXI   :label "SETUP VXI" :command DAQ-setup
      :button SHOWVXI    :label "Show VXI state" :command VXI-state
      :button HELP   :command click-help

   close-frame
}
