# vxirestore.tcl---restoring card setups

#     5 feb 1997
#    Save only task - initially created using saverestore as a base
#    restructure for TK  -  mar 31 1996  VP
#  26 Sept 95  VP
#   add Cluster
# 30-6-95  Vp
#   add CAMAC
# 9-9-94 PHO
#	Added NAM
# 5-8-93 PHO
#	Added clover
# Fri Aug  7 14:32:26 BST 1992, DB
#   on restores, report failed register writes in frame footer and give up
# Mon Aug 17 16:33:33 BST 1992, DB
#   changed patterns for save to ...[?,??]...

set debug 0
set started 0
set logging 0


set frame VXIRESTORE
set frametitle "Restoring VXI Module Setups"
set icontitle "VXI Restore"

if {[info exists env(VXIServer)] == 0} then {set vxiserver vxi1}  \
   else {set vxiserver $env(VXIServer)}

set server $vxiserver
set RegisterServer $vxiserver

loadsource common.vxi

set file ""
set savedirectory ""
set directory ""
set selection {}
set lowchannel 0
set highchannel 0
set card 0
set cardtypes {}
set tasknames {}
set cardchoice {}
set cardtype  ""

set allchans 1

set Configdir "$env(MIDASEXPBASE)"
catch {set Configdir "$env(EXPERIMENTBASE)"}

proc set-experiment {experimentdir} {
#    experiment has been joined (changed)
   global Configdir
   set Configdir $experimentdir
   start
}

proc recover-current-save-file {w} {
   global env Configdir server

   set z [catch {open [file join $Configdir [set server]_current_[set w]] r} file]
   if {$z != 0} then {set-footer "$file"; return}
   set f [gets $file]
   close $file

#    return in the format for this OS

   switch $env(platform) {
      windows {set f [midas-map-filename fromUnix $f]}
      unix    {set f [midas-map-filename fromWindows $f]}
   }

   return $f
}

proc record-current-save-file {w f} {
   global Configdir server

   set z [catch {open [file join $Configdir [set server]_current_[set w]] w} file]
   if {$z != 0} then {set-footer "$file"; return}
   puts $file "$f"
   close $file
}

proc repaint {} {
   global lowchannel highchannel card directory savedirectory allchans
   :modify LOW       :value $lowchannel
   :modify HIGH      :value $highchannel
   :modify CARD      :value $card
   :modify DIRECTORY :value $directory
   :modify FILENAME  :value $savedirectory
   :modify ALLCHANS :value $allchans
}

proc click-directory name {
   global directory
   set directory $name
   click-redisplay
}

proc click-list {v i} {
   global selection directorylist directory savedirectory file
   set selection $i

   set target [lindex $directorylist $selection]
   set name [last $target]
   if {$name == "." || $name == ".."} then  {set-footer "You cannot select $name"; return}
   if {[file isdirectory [file join $directory $name]]} then {
      set savedirectory $name
      set file ""
   }  else {
      set file $name 
   } 
   repaint
}

proc click-low v {
   global lowchannel highchannel allchans
   set lowchannel $v
   if {$lowchannel<0} then  {set lowchannel 0}
   if {$lowchannel>255} then {set lowchannel 255}
   if {$highchannel<$lowchannel} then {set highchannel $lowchannel}
   set allchans 0
   repaint
}

proc click-high v {
   global lowchannel highchannel allchans
   set highchannel $v
   if {$highchannel<0} then {set highchannel 0}
   if {$highchannel>255} then {set highchannel 255}
   if {$lowchannel>$highchannel} then {set lowchannel $highchannel}
   set allchans 0
   repaint
}

proc click-card v {
   global card cardtype cardtypes 
   set card $v
   set cardtype [lindex $cardtypes $v]
}

proc click-crate v {
   global vxicratelist vxicrate
   set vxicrate [lindex $vxicratelist $v]
}

proc select-crate c {
   global vxicratelist vxicrate
   set vxicrate $c
   :modify CRATE :value [lsearch $vxicratelist $vxicrate]
}

set directory_list_option "lt"

proc click-redisplay {} {
   global directory selection
   read-directory $directory
   paint-directory $directory
   set selection {}
   repaint
}

proc click-up {} {
  global directory savedirectory file
  set directory [directory-up $directory]
  set savedirectory ""
  set file ""
  click-redisplay
}

proc click-down {} {
  global directory selection savedirectory file
  set directory [directory-down $directory $selection]
  set savedirectory ""
  set file ""
  click-redisplay
}

set dialog_wraplength 12i
set abortflag 0

proc restoreVXIregisters {modules files} {
   global RegisterServer server
    global EGreport directory savedirectory
    global contents 
    global abortflag

   set abortmsg1 "Do you really want to abort the whole restore?\n\
                 Click on \"cancel\" if yes\n\
                 If you wish to continue with the restore then click on \"continue\""

   set abortmsg2 "\n\nRestore aborted at your request\n"
   set abortmsg3 "The VXI system is in the GO state.\n\
                 Some parameters may not restore correctly if you continue.\n\
                 It is recommended that you click on \"cancel\"  to abort, stop the VXI system and then try again.\n\
                 If you wish to continue with the restore regardless then click on \"continue\""

#    check state of VXI system
   set VXIstate [inform vxicontrol read-vxistate]
   if {$VXIstate != 0} then {
      if {[midas-confirmation "$abortmsg3"] == 0} then {append contents "$abortmsg2"; return}
   }

   set EGreport_save $EGreport
   if {$EGreport == 0} then {set EGreport -1}

   set abortflag 0
   :modify ABORT :inactive F

   if {[llength $files] == 0} then {set f $savedirectory} else {set f $files}
   insert-log "[date] Restoring VXI module setup from $f"

   set reply 0
   set mindex 0
   foreach module $modules {

#    allow chance to abort restore
      update
      if {$abortflag == 1} then {
         if {[midas-confirmation "$abortmsg1"] == 0} then {append contents "$abortmsg2"; break}
         set abortflag 0
      }
     if {$reply == 3} then {break}; #  abort restore
     if {[string range $module 0 1] == "RM"} then {continue}
#      read save file for this module
     if {[llength $files] <= $mindex} then {
        set nextfile [file join $directory $savedirectory $module]
     } else {
        set nextfile [file join $directory [lindex $files $mindex]]
     }
     incr mindex
     set z [catch {open $nextfile r} f]
     if {$z != 0} then {append contents "Unable to Restore module $module\n$f\n"; continue}
     set l [read $f]
     close $f
#      obtain crate server for this module
     set crate [VXImodule2crate $module]
     set server $crate
     set RegisterServer $crate
     append contents "Restoring module $module in VXI crate $crate\n"
#      create a list from the save file splitting at \n
     set paramlist [split $l \n]
     set index 0
     set reply 0
     foreach item $paramlist {
        if {$item == ""} then {break};    # end of file
        if {$reply >= 2} then {break};    # abort to next module
        if {$index == 0} then {
#      first item in the save file is a header 
#          set-footer "Restoring to $module using $crate [first $paramlist]"
           set-footer "Restoring to $module"
           append contents "[first $paramlist]\n"
           incr index
        } else {
           set register /$server/$module.[first $item]
           set value [string trim [string range $item [string length [first $item]] end]]
           set reply 1
           while {$reply == 1} {
              if {$EGreport == 3} then {insert-debug "restoring register $register with value $value"}
              set z [catch {eg "(register :name '$register' :value '$value')"} m]

#              set z1 [catch {eg "(register :name '$register' ?value)"} m]
#              if {"$m" != "$value"} then {insert-debug "write *$value*; read *$m*"}

              if {$z != 0} then {
#        error restoring parameter
                 set errmsg "[eg "(sys :error $m ?message)"]"
                 set msg "Error restoring register $register - $errmsg"
                 set reply [midas_dialog [self frame-name].midasDialog "Error Report for [self frame-title]" \
                     "$msg" "warning" 0 "ignore" "retry" "abort module" "abort restore"]
                 append contents "$msg\n"
                 switch $reply {
                    0    {append contents "Error ignored\n"; continue}
                    1    {append contents "Register restore being retried\n"}
                    2    {append contents "Restore for module $module aborted\n"; break}
                    3    {append contents "Whole restore aborted\n"; break}
                 } 
              } else {break}
           }
        }
     }
   }
   append contents "Restore complete\n"
   :modify ABORT :inactive T
   set EGreport $EGreport_save
}

proc click-restorebycrate {} {
   global vxicrate
   global directory savedirectory contents
   global EGreport RegisterServer server

   set contents ""

   set crates [inform vxicontrol set VXICrateList]

   if {$vxicrate == "All"} then {
     set modules ""
     foreach crate $crates {
        append modules "[inform vxicontrol set VXIModuleList($crate)]"
        append modules " "
     }
     append contents "Restored all VXI crates in configuration:- $crates\n"
   } else {
     set modules [inform vxicontrol set VXIModuleList($vxicrate)]
     append contents "Restored VXI crate $vxicrate\n"
   }
   append contents "VXI crate configuration contained modules:- $modules\n"

#    Do the restore
   set z [catch {restoreVXIregisters $modules {}} m]

#    finished - show the result
   if {$z != 0} then {midas-warning "Restore failed: $m"; return}

   if {$vxicrate == "All"} then {
      foreach crate $crates {
         set RegisterServer $crate
         set server $RegisterServer
         record-current-save-file setup [file join $directory $savedirectory]
#    set flag indicating Restore performed
         if {$EGreport == 3} then {insert-debug "setting NetVar_HasBeenRestored = 1"}
         write-register NetVar_HasBeenRestored 1
      }
       set-footer "Restored all VXI crates from save directory $savedirectory"
       insert-log "[date] Restored all VXI crates from save directory $savedirectory"
       midas-information "Restored all VXI crates from save directory $savedirectory\n\n$contents"
   } else {
         set RegisterServer $vxicrate
         set server $RegisterServer
         record-current-save-file setup [file join $directory $savedirectory]
#    set flag indicating Restore performed
         if {$EGreport == 3} then {insert-debug "setting NetVar_HasBeenRestored = 1"}
         write-register NetVar_HasBeenRestored 1
       set-footer "Restored VXI crate $vxicrate from save directory $savedirectory"
       insert-log "[date] Restored VXI crate $vxicrate from save directory $savedirectory"
       midas-information "Restored VXI crate $vxicrate from save directory $savedirectory\n\n$contents"
   }
}

proc click-restorebymodule {} {
   global cardtype cardtypes
   global allchans lowchannel highchannel
   global savedirectory contents

   set contents ""
 
   if {$cardtype == "All"} then {
     set modules [lrange $cardtypes 1 end]
     set range_start 0
     set range_end 99
     append contents "Restoring all VXI modules in configuration:- $modules\n"
   } else {
     set modules $cardtype
     if {$allchans == 0} then {
        set range_start $lowchannel
        set range_end $highchannel
        append contents "Restoring VXI module type $modules; channel range $lowchannel - $highchannel\n"
     } else {
        set range_start 0
        set range_end 99
        append contents "Restoring VXI module type $modules; all channels\n"
     }
   }

#    find all modules in the VXI configuration
   set allmodules {}
   set crates [inform vxicontrol set VXICrateList]
   foreach crate $crates {
      foreach module [inform vxicontrol set VXIModuleList($crate)] {
         lappend allmodules $module
      }
   }

#    and now expand module range
   set mm ""
   foreach module $modules {
      foreach m $allmodules {
         if {[string match $module* $m]} then {
            set index [string trimleft $m $module]
            if {$module == "TriggerMK2" || $module == "Trig"} then {
                append mm "$m "
            } else {
                if {$index >= $range_start && $index <= $range_end} then {append mm "$m " }
            }
         }
      }
   }

#  Do the restore
   set z [catch {restoreVXIregisters $mm {}} m]

#    finished - show the result
   if {$z != 0} then {midas-warning "Restore failed: $m"; return}

   set-footer "Restored VXI module(s) $cardtype from save directory $savedirectory"
   insert-log "[date] Restored VXI module(s) $cardtype from save directory $savedirectory"
   midas-information "Restored VXI module(s) $cardtype from save directory $savedirectory\n\n$contents"
}


proc click-restorebyfile {} {
   global directory file cardtype
   global lowchannel
   global contents

   if {$cardtype == "All"} then {midas-warning "You must select a module to restore to"; return}
   if {$file == ""} then {midas-warning "You must select a file to restore from"; return}

   set channel $lowchannel
   if {$cardtype == "Trig" || $cardtype == "TriggerMK2"} then {set channel ""}

   set contents "Restoring VXI module $cardtype$channel\n"

#  Do the restore
   set z [catch {restoreVXIregisters $cardtype$channel $file} m]

#    finished - show the result
   if {$z != 0} then {midas-warning "Restore failed: $m"; return}

   set-footer "Restored VXI module $cardtype$channel from save file $file"
   insert-log "[date] Restored VXI module $cardtype$channel from save file $file"
   midas-information "Restored VXI module $cardtype$channel from save file $file\n\n$contents"
}

#    mode = 0     restore crate regardless
#    mode != 0    restore only if needed
proc click-restore-current {mode} {
    global crates
    global RegisterServer server
    global vxicrate
    global directory savedirectory

   set crates [inform vxicontrol set VXICrateList]

   foreach crate $crates {
      set RegisterServer $crate
      if {$mode == 0 || [read-register NetVar_HasBeenRestored] == 0} then {
      
          set server $crate
          set f [recover-current-save-file setup]
          if {[file isdirectory $f] == 0} then {
              start
          } else {
              set vxicrate $crate
              set directory [file dirname $f]
              set savedirectory [file tail $f]
              click-restorebycrate
          }
      }
   }
}

proc click-show {what} {
   global directory savedirectory file

   if {$what == "file"} then {
     midas-show-file [file join $directory $savedirectory $file]
   } else {
     midas-show-file [file join $directory $savedirectory contents]
   }
}

proc click-filename {name} {
   global directory savedirectory file

   if {![file exists [file join $directory $name]]} then {
      midas-warning "File [file join $directory $name] does not exist"
   } else {
      if {[file isdirectory [file join $directory $name]]} then {
         set savedirectory $name
         set file ""
      } else {
         midas-warning "$name is not a directory"
      }
   }
   repaint
}

proc click arg {
   global debug frame allchans
   global allchans
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]
   if {$c==":quit"} then {return [click-quit $f]}
   switch $w {
      DIRECTORY       {click-directory $v}
      DIRECTORYLIST   {click-list $v $i}
      UP              {click-up}
      DOWN            {click-down}

      LOW             {click-low $v}
      HIGH            {click-high $v}
      ALLCHANS        {set allchans $v}
      CARD            {click-card $v}

      CRATE           {click-crate $v}

      FILENAME        {click-filename $v}
      SHOWCURRENT     {click-show save}
      SHOWFILE        {click-show file}

      RESTOREMODULE      {click-restorebymodule}
      RESTORECRATE       {click-restorebycrate}
      RESTOREFILE        {click-restorebyfile}

      REDISPLAY       {click-redisplay}
      HELP            {click-help}

      default         {insert-debug "$frame: click called with $arg"}
   }
}



proc start args {
   global env started directory cardtypes cardtype tasknames cardchoice
   global moduledefinition vxicratelist vxicrate allchans

     if {$started == 0} then {make-frame} else {open-frame}
     set started 1

     set z [catch {set directory [file join $env(EXPERIMENTBASE) VXIsetups]} m]
     if {$z != 0} then {
        midas-warning "$m\nYou should probably Join an experiment!"
        close-frame
        return
     }

     if {![file exists $directory]} then {
         midas-warning "No save directory $directory exists\n $m"
     }    
 
     set vxicratelist "All"
     append vxicratelist " " [inform vxicontrol inquire VXICrateList]

     set moduledefinition [inform vxicontrol inquire moduledefinition]
     set cardchoice {All}
     set cardtypes {All}
     set tasknames {All}
     foreach item $moduledefinition {
        if {[fourth $item] != "" && [second $item] != ""} then {
           lappend cardchoice [first $item]
           lappend cardtypes [second $item]
           lappend tasknames [third $item]
        }
     }
     set cardtype [first $cardtypes]
     :modify CARD :strings $cardchoice :value 0
     set vxicrate [first $vxicratelist]
     :modify CRATE :strings $vxicratelist :value 0

     :modify ALLCHANS :value $allchans

     click-redisplay
}

proc click-quit args {close-frame}

proc finish args {destroy-frame}

proc halt-vxi {} {inform vxicontrol DAQ-halt}
proc reset-vxi {} {inform vxicontrol DAQ-reset}
proc reset-vxi-crate {} {global vxicrate; inform vxicontrol reset-VXI $vxicrate}

proc make-frame {} {
  global env

    :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/vxi_module" \
      :icon-name "[self icon-title]" :resize F

      :panel P1 :layout T :padx 10 :pady 10
         :layout H; :row-gap 0; :col-gap 10

         :text DIRECTORY :label "Directory:" :w 60 :shifters T
         :button UP   :label Up
         :button DOWN :label Down

      :panel P2 :layout T :padx 10
         :list DIRECTORYLIST :exclusive T :rows 10 :cols 80

      :panel P3 :layout T :padx 10 :pady 10
        :layout H
        :col-gap 10; :row-gap 10
         :number LOW  :label "Low channel:"  :min 0 :max 99 :value 1 :w 4 
         :number HIGH :label "High channel:" :min 0 :max 99 :value 1 :w 4 
         :checkbox ALLCHANS  :label "All channels"
         :hskip 25
         :choice-stack CARD  :label "VXI module type:"
        :next-row T
         :choice-stack CRATE :label "VXI crate:"
        :hskip 30
         :button SHOWFILE     :label "Show Save File"
        :hskip 30
         :button VXIHALT      :label "HALT VXI"  :command halt-vxi
         :button VXIRESET     :label "RESET VXI" :command reset-vxi
         :button RESETCRATE   :label "Reset VXI Crate" :command reset-vxi-crate

        :next-row T
         :text FILENAME       :label "Save Directory:" :w 30
         :button SHOWCURRENT  :label "Show Save Contents"
         :button CURRENT      :label "Restore from Current Save"  :command "click-restore-current 0"
        :next-row T
         :col-gap 5
         :button REDISPLAY     :label Redisplay
         :button RESTOREMODULE    :label "Perform Restore by Module"
         :button RESTORECRATE     :label "Perform Restore by VXI crate"
         :button RESTOREFILE      :label "Restore file"
         :button ABORT     :command "global abortflag; set abortflag 1" :inactive T

         :button HELP  :command click-help
      
      :show T
}


